package jpopup;

import java.util.HashMap;
import java.util.Iterator;
import java.util.LinkedHashMap;

import javax.swing.JFrame;
import javax.swing.JOptionPane;
import javax.xml.parsers.ParserConfigurationException;

import jpopup.contact.ContactList;
import jpopup.history.XMLMessageHistory;
import jpopup.text.MessageFilter;

/**
 * Storage class used for store current history books, contact list, msgs wich displayed on screen and message filters
 * @author Alexey Kalukin
 *
 */
public class PopupDataStore {		
	private HashMap<String, StringBuilder> msgLists = new HashMap<String, StringBuilder>();
	private HashMap<String, XMLMessageHistory> histories = new HashMap<String, XMLMessageHistory>();
	private ContactList contactStore = null;
	private LinkedHashMap<String, MessageFilter> messageFilters = new LinkedHashMap<String, MessageFilter>();
	
	public PopupDataStore(){
		try {
			contactStore = new ContactList();
		}catch (ParserConfigurationException e) {
	//		e.printStackTrace();
			JOptionPane.showMessageDialog(new JFrame(), "Some io error with reading contact list file.\n" + e, "Error", JOptionPane.ERROR_MESSAGE);
		} 	
	}
	
	public StringBuilder getMsgList(String contactTitle) {
		return msgLists.get(contactTitle);
	}
	
	public void addMsgList(String contactTitle, StringBuilder msgList) {
		msgLists.put(contactTitle, msgList);
	}

	public void addHistory(String contactTitle, XMLMessageHistory history) {
		histories.put(contactTitle, history);
	}

	public XMLMessageHistory getHistory(String contactTitle) {
		return histories.get(contactTitle);
	}

	public ContactList getContactList() {
		return contactStore;
	}
	
	public void addMessageFilter(String filterName, MessageFilter filter) {
		messageFilters.put(filterName, filter);
	}
	
	public MessageFilter getMessageFilter(String filterName) {
		return messageFilters.get(filterName);
	}
	
	public Iterator<MessageFilter> getMessageFilterIterator() {
		return messageFilters.values().iterator();
	}
	
}
