package jpopup.gui;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.GridLayout;
import java.awt.event.ActionEvent;
import java.awt.event.KeyEvent;
import java.awt.event.KeyListener;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.net.InetAddress;
import java.net.UnknownHostException;
import java.text.ParseException;
import java.util.regex.Pattern;

import javax.swing.AbstractAction;
import javax.swing.Action;
import javax.swing.BorderFactory;
import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JMenuBar;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JTextField;
import javax.swing.JToolBar;

import jpopup.contact.Contact;
import jpopup.history.XMLMessageHistory;

import org.xml.sax.SAXException;

/**
 * This class represent window for displayed information about contact note
 * @author Alexey Kalukin
 *
 */
public class ContactWindow extends JFrame{
	private static final long serialVersionUID = -6696546355139376084L;
	private static final int INFO_PANEL_HORIZONTAL_INDENT = 5;
	private static final int INFO_PANEL_VERTICAL_INDENT = 5;
	private static final int BOTTOM_PANEL_HORIZONTAL_INDENT = 5;
	private static final int BOTTOM_PANEL_VERTICAL_INDENT = 5;
	private static final int INDENT = 5;
	
	private static final String ICQ_MASK = "^[\\d]+$";
	private static final String EMAIL_MASK = "^[\\d\\w\\.\\_]+[@]{1}[\\d\\w\\.\\_]+[\\.]{1}[\\w]{2,4}$";
	
	private Contact contact;
	
	private boolean changed = false;
	
	private Dimension defaultWindowSize = new Dimension(300, 300);
	
	private MainWindow mainFrame;
	
	private JMenuBar mainMenu;

	private JPanel mainPanel;

	private JPanel infoPanel;
	private JTextField contactAdditionalInfo;
	private JTextField contactLastName;
	private JTextField contactFirstName;
	private JTextField contactTitle;
	private JPanel bottomPanel;
	private JButton okButton;
	private JButton cancelButton;
	private JButton acceptButton;
	private JTextField contactUID;
	private JTextField contactEmail;
	private JTextField contactICQ;
	private JToolBar toolBar; 
	
	public ContactWindow(Contact contact, MainWindow mainFrame) {
		this.mainFrame = mainFrame;
		this.contact = contact;
				
		setDefaultCloseOperation(JFrame.DISPOSE_ON_CLOSE);
		addWindowListener(new ContactWindowListener(this));
		
		mainMenu = createMainMenu();
		setJMenuBar(mainMenu);
		
		getContentPane().add(createMainPanel());
		
		setPreferredSize(defaultWindowSize);
		
		ImageIcon icon = mainFrame.getUserOnlineIcon();
		String status = "online";
		
		try {
			InetAddress.getByName(contact.getUID());
		} catch (UnknownHostException e) {
			icon = mainFrame.getUserOfflineIcon();
			status = "offline";
		}
		
		setIconImage(icon.getImage());
		setTitle(contact.getTitle() + " is " + status);
	}	
	
	private JMenuBar createMainMenu() {
		return null;
	}
	
	private JPanel createMainPanel() {
		BorderLayout layout = new BorderLayout();

		mainPanel = new JPanel(layout);
		mainPanel.setBorder(BorderFactory.createEmptyBorder(INDENT, INDENT, INDENT, INDENT));		
		
		infoPanel = createInformationPanel();
		toolBar = createToolBar();
		bottomPanel = createBottomPanel();
		
		JPanel centralPanel = new JPanel(new BorderLayout());
		
		
		centralPanel.add(infoPanel, BorderLayout.CENTER);
		centralPanel.add(toolBar, BorderLayout.SOUTH);
		
		mainPanel.add(centralPanel, BorderLayout.CENTER);
		mainPanel.add(bottomPanel, BorderLayout.SOUTH);
		
		return mainPanel;
	}
	
	private JPanel createInformationPanel() {
		JPanel infoPanel = new JPanel(new GridLayout(0,2,INFO_PANEL_HORIZONTAL_INDENT, INFO_PANEL_VERTICAL_INDENT));

		contactUID = new JTextField(contact.getUID());
		contactUID.setEditable(false);
		contactUID.setBorder(BorderFactory.createLineBorder(Color.BLACK));
		
		contactTitle = new JTextField(contact.getTitle());
		contactTitle.addKeyListener(new PlainTextKeyListener(this));
		contactTitle.setBorder(BorderFactory.createLineBorder(Color.BLACK));
		
		contactFirstName = new JTextField(contact.getFirstName());
		contactFirstName.addKeyListener(new PlainTextKeyListener(this));
		contactFirstName.setBorder(BorderFactory.createLineBorder(Color.BLACK));
		
		contactLastName = new JTextField(contact.getLastName());
		contactLastName.addKeyListener(new PlainTextKeyListener(this));
		contactLastName.setBorder(BorderFactory.createLineBorder(Color.BLACK));
		
		contactEmail = new JTextField(contact.getEmail());
		contactEmail.addKeyListener(new EmailTextKeyListener(this));
		contactEmail.setBorder(BorderFactory.createLineBorder(Color.BLACK));
		
		contactICQ = new JTextField(contact.getICQ());		
		contactICQ.addKeyListener(new ICQTextKeyListener(this));
		contactICQ.setBorder(BorderFactory.createLineBorder(Color.BLACK));
		
		contactAdditionalInfo = new JTextField(contact.getAdditionalInfo());
		contactAdditionalInfo.addKeyListener(new PlainTextKeyListener(this));
		contactAdditionalInfo.setBorder(BorderFactory.createLineBorder(Color.BLACK));
				
		infoPanel.add(new JLabel("UID:"));
		infoPanel.add(contactUID);
		infoPanel.add(new JLabel("Title:"));
		infoPanel.add(contactTitle);
		infoPanel.add(new JLabel("First Nam/e:"));
		infoPanel.add(contactFirstName);
		infoPanel.add(new JLabel("Last Name:"));
		infoPanel.add(contactLastName);
		infoPanel.add(new JLabel("E-mail:"));
		infoPanel.add(contactEmail);
		infoPanel.add(new JLabel("ICQ:"));
		infoPanel.add(contactICQ);
		infoPanel.add(new JLabel("Additional info:"));
		infoPanel.add(contactAdditionalInfo);
		
		return infoPanel;
	}
	
	private JToolBar createToolBar() {
		JToolBar mainToolBar = new JToolBar(JToolBar.HORIZONTAL);
		mainToolBar.setLayout(new FlowLayout(FlowLayout.LEFT));
		
		mainToolBar.add(new ShowHistoryAction());
		mainToolBar.add(new PingAction(this));
		
		return mainToolBar;
	}
	
	
	private JPanel createBottomPanel() {
		JPanel bottomPanel = new JPanel(new FlowLayout(FlowLayout.RIGHT, BOTTOM_PANEL_HORIZONTAL_INDENT, BOTTOM_PANEL_VERTICAL_INDENT));
		
		okButton = new JButton("OK");
		okButton.addActionListener(new OkAction(this));		
		cancelButton = new JButton("Cancel");
		cancelButton.addActionListener(new CancelAction());
		acceptButton = new JButton("Accept");
		acceptButton.addActionListener(new AcceptAction());
		acceptButton.setEnabled(false);
		
		bottomPanel.add(okButton);
		bottomPanel.add(cancelButton);
		bottomPanel.add(acceptButton);
		
		return bottomPanel;
	}
	
	private void saveChanges() {
		contact.setTitle(contactTitle.getText());
		contact.setFirstName(contactFirstName.getText());
		contact.setLastName(contactLastName.getText());
		if (validEmail(contactEmail.getText())) {
			contact.setEmail(contactEmail.getText());
		} else {
			JOptionPane
			.showMessageDialog(null, "Wrong e-mail address: " + contactEmail.getText(), "Error",JOptionPane.WARNING_MESSAGE);
			changed = false;
			acceptButton.setEnabled(false);
			return;
		}
		if (validICQ(contactICQ.getText())) {
			contact.setICQ(contactICQ.getText());
		} else {
			JOptionPane
			.showMessageDialog(null, "Wrong ICQ uid: " + contactICQ.getText(), "Error",JOptionPane.WARNING_MESSAGE);
			changed = false;
			acceptButton.setEnabled(false);
			return;
		}
		contact.setAdditionalInfo(contactAdditionalInfo.getText());
		
		changed = false;
		acceptButton.setEnabled(false);
	}
	
	private boolean validEmail(String email) {
		boolean valid = true;
		if (!"".equals(email)) {
			valid = Pattern.matches(EMAIL_MASK, email) ;
		}
		
		return valid;
	}
	
	private boolean validICQ(String icq) {
		boolean valid = true;
		if (!"".equals(icq)) {
			valid = Pattern.matches(ICQ_MASK, icq);
		}
		
		return valid;
	}
	
	protected void setChanged(boolean changed) {
		this.changed = changed;
	}
	

	protected boolean isChanged() {
		return changed;
	}


	private class ContactWindowListener extends WindowAdapter {

		private JFrame mainFrame;
		public ContactWindowListener(JFrame mainFrame) {
			this.mainFrame = mainFrame;
		}

		public void windowClosing(WindowEvent arg0) {
			mainFrame.setDefaultCloseOperation(JFrame.DISPOSE_ON_CLOSE);
			if (isChanged()) {
				int userAnswer = JOptionPane.showConfirmDialog(mainFrame, "Do you wnat to save changes?", "Save changes", JOptionPane.YES_NO_CANCEL_OPTION);		
				switch (userAnswer) {
				case JOptionPane.YES_OPTION:
					saveChanges();
				case JOptionPane.NO_OPTION:
					dispose();
					break;
				case JOptionPane.CANCEL_OPTION:
					mainFrame.setDefaultCloseOperation(JFrame.DO_NOTHING_ON_CLOSE);
					break;
				}				
			} else {			
				dispose();
			}
			
		}
	}
	
	private class InfoKeyListener implements KeyListener {
		private ContactWindow contactWindow;
		
		public InfoKeyListener(ContactWindow contactWindow) {
			this.contactWindow = contactWindow;
		}
		public void keyPressed(KeyEvent arg0) {
		}

		public void keyReleased(KeyEvent arg0) {			
		}

		public void keyTyped(KeyEvent arg0) {
			contactWindow.setChanged(true);
			acceptButton.setEnabled(true);
		}
		protected void setContactWindow(ContactWindow contactWindow) {
			this.contactWindow = contactWindow;
		}
		protected ContactWindow getContactWindow() {
			return contactWindow;
		}
		
	}
	
	private class PlainTextKeyListener extends InfoKeyListener {
		
		public PlainTextKeyListener(ContactWindow contactWindow) {
			super(contactWindow);
		}
	
	}
	
	private class EmailTextKeyListener extends InfoKeyListener {

		public EmailTextKeyListener(ContactWindow contactWindow) {
			super(contactWindow);
		}
		
		public void keyReleased(KeyEvent arg0) {
			if (!getContactWindow().contactEmail.getText().equals("")) {
				if (!validEmail(getContactWindow().contactEmail.getText())) {
					getContactWindow().contactEmail.setBorder(BorderFactory.createLineBorder(Color.RED));
				} else {
					getContactWindow().contactEmail.setBorder(BorderFactory.createLineBorder(Color.BLACK));
				}
				getContactWindow().setChanged(true);
				acceptButton.setEnabled(true);
			}
		}
		
	}
	
	private class  ICQTextKeyListener extends InfoKeyListener {

		public  ICQTextKeyListener(ContactWindow contactWindow) {
			super(contactWindow);
		}
		
		public void keyReleased(KeyEvent arg0) {
			if (!getContactWindow().contactICQ.getText().equals("")) {
				if (!validICQ(getContactWindow().contactICQ.getText())) {
					getContactWindow().contactICQ.setBorder(BorderFactory.createLineBorder(Color.RED));
				} else {
					getContactWindow().contactICQ.setBorder(BorderFactory.createLineBorder(Color.BLACK));
				}
				getContactWindow().setChanged(true);
				acceptButton.setEnabled(true);
			}
		}
		
	}
	
	
	private class OkAction extends AbstractAction {

		private static final long serialVersionUID = -7709085984114138322L;
		private JFrame mainFrame;
		
		public OkAction(JFrame mainFrame) {
			this.mainFrame = mainFrame;
		}
		
		public void actionPerformed(ActionEvent arg0) {
			if (isChanged()) {
				int userAnswer = JOptionPane.showConfirmDialog(mainFrame, "Do you wnat to save changes?", "Save changes", JOptionPane.YES_NO_CANCEL_OPTION);		
				switch (userAnswer) {
				case JOptionPane.YES_OPTION:
					saveChanges();
				case JOptionPane.NO_OPTION:
					dispose();
					break;
				}									
			} else {			
				dispose();
			}
		}
		
	}
	
	private class CancelAction extends AbstractAction {

		private static final long serialVersionUID = 8677512804284591806L;

		public void actionPerformed(ActionEvent arg0) {
			dispose();
		}
		
	}
	
	private class AcceptAction extends AbstractAction {

		private static final long serialVersionUID = 6833607216215962051L;

		public void actionPerformed(ActionEvent arg0) {
			saveChanges();
		}
		
	}
	
	private class ShowHistoryAction extends AbstractAction {

		private static final long serialVersionUID = -6895835367219260049L;
		private static final String SHOW_HYSOTRY_BUTTON_NAME = "History";
		
		public ShowHistoryAction() {
			putValue(Action.NAME, SHOW_HYSOTRY_BUTTON_NAME);
			putValue(Action.SMALL_ICON, mainFrame.getHistoryIcon());
		}
		
		public void actionPerformed(ActionEvent arg0) {
			XMLMessageHistory history = null;
			try {
				history = (XMLMessageHistory)contact.getHistory();
				history.getHistoryFromXMLFile();
			} catch (SAXException e1) {
		//		e1.printStackTrace();
				JOptionPane.showMessageDialog(null,
						"Parse XML error. Can't read history xml file "
								+ history.getFileName() + ".\n" + e1, "Error",
						JOptionPane.ERROR_MESSAGE);
			} catch (FileNotFoundException e) {
	//			e.printStackTrace();
				JOptionPane.showMessageDialog(null,
						"Can't find XML file " + history.getFileName()
								+ " for reading history.\n" + e, "Error",
						JOptionPane.ERROR_MESSAGE);
			} catch (IOException e) {
		//		e.printStackTrace();
				JOptionPane
						.showMessageDialog(null,
								"IO Error with history xml file "
										+ history.getFileName() + " \n" + e, "Error",
								JOptionPane.ERROR_MESSAGE);
			} catch (ParseException e) {
				JOptionPane.showMessageDialog(null, "Parse XML file "
						+ history.getFileName() + ".\n" + e, "Error",
						JOptionPane.ERROR_MESSAGE);
		//		e.printStackTrace();
			}
			HistoryWindow hw = new HistoryWindow(contact.getHistory(), mainFrame);
			hw.pack();
			hw.setVisible(true);						
		}
		
	}
	
	private class PingAction extends AbstractAction {
		private static final long serialVersionUID = -1732991524960531143L;

		private ContactWindow contactFrame;
		
		public PingAction(ContactWindow contactFrame) {
			putValue(AbstractAction.SMALL_ICON, mainFrame.getPingIcon());
			this.contactFrame = contactFrame;
		}
		
		public void actionPerformed(ActionEvent e) {
			
			InetAddress address = null;
			try {
				address = InetAddress.getByName(contact.getUID());
			} catch (UnknownHostException e1) {
			}
			
			ImageIcon statusIcon;
			String status;
			if (address != null) {
				JOptionPane.showMessageDialog(null, "Computer \"" + contact.getUID() + "\" is ONLINE now.\n" 
						+ "IP :" + address.getHostAddress(), "ONLINE", JOptionPane.INFORMATION_MESSAGE, mainFrame.getUserOnlineIcon());
				statusIcon = mainFrame.getUserOnlineIcon();
				status = "online";
				
			} else {
				JOptionPane.showMessageDialog(null, "Computer \"" + contact.getUID() + "\" is OFFLINE now.", "OFFLINE", JOptionPane.INFORMATION_MESSAGE, mainFrame.getUserOfflineIcon());
				statusIcon = mainFrame.getUserOfflineIcon();
				status = "offline";
			}
			contactFrame.setTitle(contact.getTitle() + " is " + status);
			contactFrame.setIconImage(statusIcon.getImage());
		}

	}

}
