package jpopup.gui;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.event.KeyAdapter;
import java.awt.event.KeyEvent;
import java.util.ArrayList;

import javax.swing.BorderFactory;
import javax.swing.JEditorPane;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JMenuBar;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTextField;
import javax.swing.text.html.HTMLEditorKit;

import jpopup.history.Message;
import jpopup.history.MessageHistory;

/**
 * This class used to display message history of some contact or group
 * @author Alexey Kalukin
 *
 */
public class HistoryWindow extends JFrame{

	private static final long serialVersionUID = -6327874363915495693L;

	private static final String HYSTORY_TITLE = "Message history of ";
	private final int INDENT = 5;
	
	private String startTags = "<html><body>";
	private String finishTags = "</html></body>";

	private int preferedWidth = 500;
	private int preferedHeight = 300;
	
	private MessageHistory history;
	private ArrayList<String> upHistory = new ArrayList<String>();

	private JMenuBar mainMenu;

	private JPanel mainPanel;

	private JPanel historyListPanel;
	
	private MainWindow mainFrame;

	private JTextField filterField;

	private JPanel filterPanel;

	private StringBuilder msgList;

	private JEditorPane chatTextPane;

	private JLabel filterMsgCountLabel;

	private int visibleMsgsCount;

	private JPanel topPanel;
	
	public HistoryWindow(MessageHistory history, MainWindow mainFrame) {
		
		this.mainFrame = mainFrame;
		this.history = history;
		for (Message msg: history) {
			upHistory.add(msg.getText().toUpperCase());
		}
			
		setTitle(HYSTORY_TITLE + history.getContactTitle());
		
		setDefaultCloseOperation(JFrame.DISPOSE_ON_CLOSE);		
		
		mainMenu = createMainMenu();
		setJMenuBar(mainMenu);
		
		mainPanel = createMainPanel();
		getContentPane().add(mainPanel);
		
		setPreferredSize(new Dimension(preferedWidth, preferedHeight));
		setIconImage(mainFrame.getHistoryIcon().getImage());
	}	
	
	private JMenuBar createMainMenu() {
		return null;
	}
	
	private JPanel createMainPanel() {
		JPanel mainPanel = new JPanel(new BorderLayout(INDENT, INDENT));
		mainPanel.setBorder(BorderFactory.createEmptyBorder(INDENT, INDENT, INDENT, INDENT));	
		
		historyListPanel = createHistoryListPanel();
		filterPanel = createFilterPanel();
		topPanel = createTopPanel();
		
		mainPanel.add(historyListPanel, BorderLayout.CENTER);
		mainPanel.add(filterPanel, BorderLayout.SOUTH);
		mainPanel.add(topPanel, BorderLayout.NORTH);
		
		return mainPanel;
	}
	
	protected JPanel createHistoryListPanel() {
		JPanel historyListPanel = new JPanel(new BorderLayout());
				
		chatTextPane = new JEditorPane();
		chatTextPane.setEditable(false);
		chatTextPane.setEditorKit(new HTMLEditorKit());
		chatTextPane.setText(startTags + finishTags);
		chatTextPane.addHyperlinkListener(mainFrame.getMainHyperlinkClickEvent());

		msgList = new StringBuilder();
		for (Message msg: history) {									
			String newMSGText = mainFrame.preparingTextMsg(msg.getText());
			msgList.append(mainFrame.addTagsToMsg(newMSGText, msg.getSender(), msg.getTime()));
			visibleMsgsCount++;			
		}
		chatTextPane.setText(startTags + msgList + finishTags);
		chatTextPane.setSelectionStart(
						chatTextPane.getText().length() - 1);		

		JScrollPane chatScrollpane = new JScrollPane(chatTextPane);
		chatScrollpane.setHorizontalScrollBarPolicy(JScrollPane.HORIZONTAL_SCROLLBAR_AS_NEEDED);
		chatScrollpane.setVerticalScrollBarPolicy(JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED);		
		
		historyListPanel.add(chatScrollpane);
		
		return historyListPanel;
	}
	
	private JPanel createTopPanel() {
		JPanel topPanel = new JPanel(new FlowLayout(FlowLayout.LEFT, 0, 0));
		
		JLabel messageCountLabel = new JLabel("Message count: ");
		
		filterMsgCountLabel = new JLabel(String.valueOf(visibleMsgsCount));
		
		topPanel.add(messageCountLabel);
		topPanel.add(filterMsgCountLabel);
		
		return topPanel;
	}
	
	private JPanel createFilterPanel() {
		JPanel filterPanel = new JPanel(new BorderLayout(INDENT, INDENT));
		
		JLabel filterLabel = new JLabel("Filter:");
		
		filterField = new JTextField();
		filterField.setBorder(BorderFactory.createLineBorder(Color.BLACK));
		filterField.addKeyListener(new FilterKeyListener(this));
		
		filterPanel.add(filterLabel, BorderLayout.WEST);
		filterPanel.add(filterField, BorderLayout.CENTER);
		
		return filterPanel;
	}
	
	public void setFilterPanel(JPanel filterPanel) {
		this.filterPanel = filterPanel;
	}

	public JPanel getFilterPanel() {
		return filterPanel;
	}

	public int makeFilterMsgs(String filter) {
		visibleMsgsCount = 0;
		msgList = new StringBuilder();
		String upFilter = filter.toUpperCase();
		int counter = 0;
		for (Message msg: history) {			
			if (upHistory.get(counter).contains(upFilter)) {
				String newMSGText = mainFrame.preparingTextMsg(msg.getText());
				msgList.append(mainFrame.addTagsToMsg(newMSGText, msg.getSender(), msg.getTime()));
				visibleMsgsCount++;
			}
			counter++;
		}
		chatTextPane.setText(startTags + msgList + finishTags);
		chatTextPane.setSelectionStart(
						chatTextPane.getText().length() - 1);
		filterMsgCountLabel.setText(String.valueOf(visibleMsgsCount));
		
		return visibleMsgsCount;
	}
	
	private class FilterKeyListener extends KeyAdapter {		
		private HistoryWindow historyWindow;
		
		public FilterKeyListener(HistoryWindow historyWindow) {
			this.historyWindow = historyWindow;
		}
		
		public void keyReleased(KeyEvent arg0) {
			int msgsCount = historyWindow.makeFilterMsgs(filterField.getText()); 
			if (msgsCount == 0) {
				filterField.setBorder(BorderFactory.createLineBorder(Color.RED));
			} else {
				filterField.setBorder(BorderFactory.createLineBorder(Color.BLACK));
			}
						
		}
	}	
}
