package jpopup.gui;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.Point;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.InputEvent;
import java.awt.event.KeyEvent;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.UnsupportedEncodingException;
import java.net.InetAddress;
import java.net.SocketException;
import java.net.UnknownHostException;
import java.text.DateFormat;
import java.text.FieldPosition;
import java.text.ParseException;
import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.Iterator;
import java.util.LinkedHashMap;
import java.util.Map.Entry;

import javax.swing.AbstractAction;
import javax.swing.DefaultListModel;
import javax.swing.Icon;
import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JComponent;
import javax.swing.JEditorPane;
import javax.swing.JFrame;
import javax.swing.JList;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JPopupMenu;
import javax.swing.JScrollPane;
import javax.swing.JSplitPane;
import javax.swing.JTabbedPane;
import javax.swing.JTextArea;
import javax.swing.JTextField;
import javax.swing.JToolBar;
import javax.swing.KeyStroke;
import javax.swing.ListSelectionModel;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;
import javax.swing.text.html.HTMLEditorKit;
import javax.xml.parsers.ParserConfigurationException;
import javax.xml.transform.TransformerException;

import jpopup.PopupDataStore;
import jpopup.contact.Contact;
import jpopup.gui.actions.AboutAction;
import jpopup.gui.actions.AddContactAction;
import jpopup.gui.actions.AddContactFromTabAction;
import jpopup.gui.actions.ChooseDeviceAction;
import jpopup.gui.actions.CloseAllTabsAction;
import jpopup.gui.actions.CloseApplicationAction;
import jpopup.gui.actions.CloseOthersTabsAction;
import jpopup.gui.actions.CloseTabAction;
import jpopup.gui.actions.ShowHistoryAction;
import jpopup.gui.listeners.ChatHyperlinkClickListener;
import jpopup.gui.xml.SettingsManager;
import jpopup.gui.xml.StateManager;
import jpopup.gui.xml.StateManager.Tab;
import jpopup.history.Message;
import jpopup.history.MessageHistory;
import jpopup.history.XMLMessageHistory;
import jpopup.popupengine.LinuxPopupCatcher;
import jpopup.popupengine.PopupCatcher;
import jpopup.popupengine.PopupListener;
import jpopup.popupengine.PopupMSG;
import jpopup.popupengine.PopupSMBMessage;
import jpopup.popupengine.WindowsPopupDaemon;
import jpopup.popupengine.text.JPopupEngine;
import jpopup.popupengine.text.TextPopupMSG;
import jpopup.popupengine.text.WindowsPopupCatcher;
import jpopup.text.MessageFilter;
import net.sourceforge.jpcap.capture.CaptureDeviceLookupException;
import net.sourceforge.jpcap.capture.CaptureDeviceOpenException;
import net.sourceforge.jpcap.capture.PacketCapture;

import org.xml.sax.SAXException;

/**
 * Class wich invoke all main graphics resources and main window
 * @author Alexey Kalukin
 *
 */
public class MainWindow {
	private static final String WINDOWS_PATTERN = "Windows";

	private static final String GENERAL_TAB_NAME = "*";	
	private String myName = System.getenv("USERDOMAIN");
	private static final String DEFAULT_TITLE = "GPopup - WinPopup messenger";
	private static final int INDENT = 5;

	private String stateFileName = "state.xml";
	private String contactListFileName = "contacts.xml";
	private String settingsFileName = "settings.xml";
	private String iconsDirectory = "icons/default/";

	private String msgHeaderColor = "#EEEEEE";
	private String startTags = "<html><body>";	
	private String finishTags = "</html></body>";
	private String statrSMGPattern = "<!--StartMSG-->";	

	private final int TAB_ADD_CONTACT_INDEX = 0;

	private Dimension defaultWindowSize = new Dimension(640, 400);
	private Point defaultWindowLocation = new Point(0, 0);
	private int defaultMainHorizontalDividerPosition = 450;
	private int defaultMainVerticlaDividerPosition = 220;
		
	private Font defaultFont = new Font("Arial", Font.BOLD, 12);

	private JFrame mainFrame;
	private JPanel mainPanel;
	private JPanel inputPanel;
	private JPanel contactPanel;
	private JPanel addContactPanel;
	private JScrollPane inputScrollPane;
	private JScrollPane contactScrollPanel;
	private JTabbedPane chatTabbedPane;
	private JSplitPane centerPanel;
	private JSplitPane mainSplitPanel;
	private JMenuBar mainMenu;
	private JList contactList;
	private JTextField addContactString;
	private JTextArea inputTextArea;
	private JButton sendButton;
	private JButton addContactButton;
	private JPopupMenu contactListMenu;
	private JPopupMenu tabMenu;	
	private JToolBar inputToolBar;
	private JMenu filtersMenu;
	private JMenu devicesMenu;
	private DefaultListModel contactListModel = new DefaultListModel();

	private SendAction sendAction;
	private ChatHyperlinkClickListener mainHyperlinkClickEvent;

	private int visibleMsgsCount = 20;
	private int notificationCount = 0;	

	private HashMap<String, JEditorPane> chatTextList = new HashMap<String, JEditorPane>();
	private HashMap<String, Integer> visibleCount = new HashMap<String, Integer>();
	private HashMap<String, JScrollPane> scrollPanes = new HashMap<String, JScrollPane>();

	private  static LinkedHashMap<String, String> escapeXmls = new LinkedHashMap<String, String>();
	private PopupDataStore popupDataStore = new PopupDataStore();
	private SettingsManager settingsManager;
	
	private JPopupEngine jPopup;
	
	private ImageIcon noNewMSGIcon;
	private ImageIcon newMSGIcon;
	private ImageIcon getMSGMainIcon;
	private ImageIcon noMSGMainIcon;
	private ImageIcon userOnlineIcon;
	private ImageIcon userOfflineIcon;
	private ImageIcon historyIcon;
	private ImageIcon pingIcon;
	private ImageIcon aboutIcon;
	private ImageIcon exitIcon;
	private ImageIcon deviceIcon;
	private ImageIcon aboutItemIcon;
	private ImageIcon addUserIcon;
	private ImageIcon sendIcon;
	private ImageIcon viewContactInfoIcon;
	private ImageIcon removeContactIcon;
	private ImageIcon addTabIcon;
	private ImageIcon closeTabIcon;
	private ImageIcon closeAllTabsIcon;

	public MainWindow(PopupDataStore popupDataStore) {
		initData(popupDataStore);		
		createAndShowGUI();
		readSettings();
		
		PopupCatcher catcher = null;
		if (isWindows()) {
			catcher = chooseEthernetDevice(false, true);
		} else {
			try {
				catcher = new LinuxPopupCatcher();
			} catch (SocketException e) {
				// e.printStackTrace();
				JOptionPane.showMessageDialog(mainFrame,
						"Can't open socket for broadcast.\n" + e, "Error",
						JOptionPane.ERROR_MESSAGE);
			}
		}		
		if (catcher != null) {
			jPopup = new JPopupEngine(catcher);
			jPopup.addPopupListener(new MyPopupListener());
			jPopup.startListen();			
		}
	}
	
	private void initData(PopupDataStore popupDataStore) {
		this.popupDataStore = popupDataStore;		
		initImages();				
		initEscapeXmls();
		
		sendAction = new SendAction();
		mainHyperlinkClickEvent = new ChatHyperlinkClickListener(this);		
	}
	
	private void initImages() {
		File imgDir = new File(iconsDirectory);
		if (!imgDir.exists()) {
			imgDir.mkdir();
		}
		
		newMSGIcon = new ImageIcon(iconsDirectory + "getMSG.gif");
		noNewMSGIcon = new ImageIcon(iconsDirectory + "noMSG.gif");
		getMSGMainIcon = new ImageIcon(iconsDirectory + "getMSG.gif");
		noMSGMainIcon = new ImageIcon(iconsDirectory + "noMSG.gif");
		userOfflineIcon = new ImageIcon(iconsDirectory + "userOffline.gif");
		userOnlineIcon = new ImageIcon(iconsDirectory + "userOnline.gif");
		historyIcon = new  ImageIcon(iconsDirectory + "history.gif");
		pingIcon = new  ImageIcon(iconsDirectory + "ping.gif");
		aboutIcon = new ImageIcon(iconsDirectory + "about.gif");
		exitIcon = new ImageIcon(iconsDirectory + "exit.gif");
		deviceIcon = new ImageIcon(iconsDirectory + "device.gif");
		aboutItemIcon = new ImageIcon(iconsDirectory + "aboutItem.gif");
		addUserIcon = new ImageIcon(iconsDirectory + "addUser.gif");
		sendIcon = new ImageIcon(iconsDirectory + "send.gif");
		viewContactInfoIcon = new ImageIcon(iconsDirectory + "viewContact.gif");
		removeContactIcon = new ImageIcon(iconsDirectory + "removeUser.gif");
		addTabIcon = new ImageIcon(iconsDirectory + "addTab.gif");
		closeTabIcon = new ImageIcon(iconsDirectory + "closeTab.gif");
		closeAllTabsIcon = new ImageIcon(iconsDirectory + "closeAllTabs.gif");
	}
	
	private void initEscapeXmls() {
		escapeXmls.put("[&]{1}", "&amp;");
		escapeXmls.put("[<]{1}", "&lt;");
		escapeXmls.put("[>]{1}", "&gt;");
		escapeXmls.put("[$]{1}", "\\\\$");
		escapeXmls.put("[%]{1}", "\\\\%");
		escapeXmls.put("[\n]{1}", "<br>");
		escapeXmls.put("[\r]{1}", "");				
	}
	
	private void createAndShowGUI() {
		mainFrame = new JFrame(DEFAULT_TITLE);
		mainFrame.setIconImage(noMSGMainIcon.getImage());
		mainFrame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
		mainFrame.addWindowListener(new MainFrameListener(this));

		mainMenu = createMainMenu();
		mainFrame.getContentPane().add(createMainPanel());
		mainFrame.setJMenuBar(mainMenu);

		mainFrame.setLocation(defaultWindowLocation);
		mainFrame.setPreferredSize(defaultWindowSize);
		
		int currentTabIndex = chatTabbedPane.getSelectedIndex();
		if (currentTabIndex > -1) {
			mainFrame.setTitle(chatTabbedPane.getTitleAt(currentTabIndex));
		}
		createTabPopupMenu();
		
		mainFrame.pack();
		mainFrame.setVisible(true);
	}

	private JPanel createMainPanel() {
		mainPanel = new JPanel(new BorderLayout(INDENT, INDENT));
		
		inputPanel = createInputPanel();
		chatTabbedPane = createChatPanel();
		chatTabbedPane.addChangeListener(new TabChangeListener());

		contactPanel = createContactListPanel();
		centerPanel = new JSplitPane(JSplitPane.HORIZONTAL_SPLIT,
				chatTabbedPane, contactPanel);
		centerPanel.setOneTouchExpandable(true);
		centerPanel.setDividerLocation(defaultMainHorizontalDividerPosition);
		centerPanel.setResizeWeight(1);

		mainSplitPanel = new JSplitPane(JSplitPane.VERTICAL_SPLIT, centerPanel,
				inputPanel);
		mainSplitPanel.setDividerLocation(defaultMainVerticlaDividerPosition);
		mainSplitPanel.setOneTouchExpandable(true);
		mainSplitPanel.setResizeWeight(1);

		mainPanel.add(mainSplitPanel, BorderLayout.CENTER);

		return mainPanel;
	}

	private JTabbedPane createChatPanel() {
		JTabbedPane chatTabbedPane = new JTabbedPane();
		
		chatTabbedPane.addMouseListener(new TabMouseListener());
		chatTabbedPane.setInheritsPopupMenu(true);
			
		return chatTabbedPane;
	}

	private JPanel createContactListPanel() {
		contactList = new JList(contactListModel);
		contactList.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
		contactList.addMouseListener(new ContactListMouseListener());
		contactList.setInheritsPopupMenu(true);
		contactList.add(createContactListPopupMenu());
		contactList.setFont(defaultFont);
		contactScrollPanel = new JScrollPane(contactList);
		
		addContactButton = new JButton(addUserIcon);
		addContactButton.addActionListener(new AddContactAction(this));
		addContactButton.setToolTipText("Add new user to contact list");

		addContactString = new JTextField();
		addContactString.addActionListener(new AddContactAction(this));
		addContactString.setFont(defaultFont);		

		addContactPanel = new JPanel(new BorderLayout(INDENT, INDENT));
		addContactPanel.add(addContactString, BorderLayout.CENTER);
		addContactPanel.add(addContactButton, BorderLayout.EAST);

		JPanel contactArea = new JPanel(new BorderLayout(INDENT, INDENT));
		contactArea.add(contactScrollPanel, BorderLayout.CENTER);
		contactArea.add(addContactPanel, BorderLayout.NORTH);

		return contactArea;
	}

	private JMenuBar createMainMenu() {
		JMenuBar mainMenu = new JMenuBar();

		JMenu fileMenu = new JMenu("File");
		fileMenu.setMnemonic(KeyEvent.VK_F);

		JMenuItem exitItem = new JMenuItem("Exit",KeyEvent.VK_E);
		exitItem.addActionListener(new CloseApplicationAction(this));
		exitItem.setIcon(exitIcon);
		exitItem.setToolTipText("Close application");
		fileMenu.add(exitItem);		

		filtersMenu = new JMenu("Filters");
		filtersMenu.setMnemonic(KeyEvent.VK_I);
		
		devicesMenu = new JMenu("Devices");
		devicesMenu.setMnemonic(KeyEvent.VK_D);
		
		JMenuItem chooseDeviceItem = new JMenuItem("Choose device", KeyEvent.VK_C);
		chooseDeviceItem.addActionListener(new ChooseDeviceAction(this));
		chooseDeviceItem.setIcon(deviceIcon);
		chooseDeviceItem.setToolTipText("Choose ethernet device for receiving messages");
		
		devicesMenu.add(chooseDeviceItem);
		
		JMenu helpMenu = new JMenu("Help");
		helpMenu.setMnemonic(KeyEvent.VK_H);
				
		JMenuItem aboutItem = new JMenuItem("About", KeyEvent.VK_A);
		aboutItem.addActionListener(new AboutAction(this));
		aboutItem.setIcon(aboutItemIcon);
		aboutItem.setToolTipText("Some information about creators");
		helpMenu.add(aboutItem);

		mainMenu.add(fileMenu);
		mainMenu.add(filtersMenu);
		mainMenu.add(devicesMenu);
		mainMenu.add(helpMenu);
		return mainMenu;
	}

	private JPanel createInputPanel() {
		JPanel inputPanel = new JPanel(new BorderLayout());

		inputTextArea = new JTextArea();
		inputTextArea.getInputMap(JComponent.WHEN_IN_FOCUSED_WINDOW)
				.put(
						KeyStroke.getKeyStroke(KeyEvent.VK_ENTER,
								InputEvent.CTRL_MASK), sendAction);

		inputScrollPane = new JScrollPane(inputTextArea);

		JPanel sendButtonPanel = new JPanel(new BorderLayout());
		
		sendButton = new JButton(sendAction);		
		sendButton.setIcon(sendIcon);
		sendButtonPanel.add(sendButton, BorderLayout.CENTER);

		inputToolBar = createInputToolbar();

		inputPanel.add(inputScrollPane, BorderLayout.CENTER);
		inputPanel.add(sendButtonPanel, BorderLayout.EAST);
		inputPanel.add(inputToolBar, BorderLayout.NORTH);

		return inputPanel;
	}

	private JToolBar createInputToolbar() {		
		JToolBar inputToolBar = new JToolBar();		
		
		inputToolBar.add(new ShowHistoryAction(this));
		inputToolBar.add(new ViewContactInfoAction(this, ViewContactInfoAction.TOOLBAR));
		inputToolBar.add(new PingContactAction(PingContactAction.TOOLBAR));
		inputToolBar.addSeparator();		

		return inputToolBar;
	}
	
	private JPopupMenu createTabPopupMenu() {
		tabMenu = new JPopupMenu();

		JMenuItem addContactItem = new JMenuItem("Add to contact list");
		addContactItem.addActionListener(new AddContactFromTabAction(this));
		addContactItem.setIcon(addUserIcon);

		JMenuItem closeTabItem = new JMenuItem("Close tab");
		closeTabItem.addActionListener(new CloseTabAction(this));
		closeTabItem.setIcon(closeTabIcon);

		JMenuItem closeAllTabsItem = new JMenuItem("Close all tabs");
		closeAllTabsItem.addActionListener(new CloseAllTabsAction(this));
		closeAllTabsItem.setIcon(closeAllTabsIcon);

		JMenuItem closeAllTabsButThis = new JMenuItem("Close all tabs but this");
		closeAllTabsButThis.addActionListener(new CloseOthersTabsAction(this));

		tabMenu.add(addContactItem);
		tabMenu.addSeparator();
		tabMenu.add(closeTabItem);
		tabMenu.add(closeAllTabsButThis);
		tabMenu.add(closeAllTabsItem);

		return tabMenu;
	}

	private JPopupMenu createContactListPopupMenu() {
		contactListMenu = new JPopupMenu();
		
		JMenuItem removeContactItem = new JMenuItem("Remove contact");
		removeContactItem.addActionListener(new RemoveContactAction());
		removeContactItem.setIcon(removeContactIcon);

		JMenuItem veiwInfoItem = new JMenuItem("View info");
		veiwInfoItem.addActionListener(new ViewContactInfoAction(this, ViewContactInfoAction.CONTACT_LIST));
		veiwInfoItem.setIcon(viewContactInfoIcon);

		JMenuItem pingContactitem = new JMenuItem("Ping contact");
		pingContactitem.addActionListener(new PingContactAction(PingContactAction.CONTACT_LIST));
		pingContactitem.setIcon(pingIcon);

		JMenuItem createTabItem = new JMenuItem("Create chat tab");
		createTabItem.addActionListener(new CreateTabAction());
		createTabItem.setIcon(addTabIcon);

		contactListMenu.add(veiwInfoItem);
		contactListMenu.add(createTabItem);
		contactListMenu.add(pingContactitem);
		contactListMenu.addSeparator();
		contactListMenu.add(removeContactItem);

		return contactListMenu;
	}

	public boolean isWindows() {
		return System.getProperty("os.name").contains(WINDOWS_PATTERN);
	}

	public void saveWindowState() {
		try {
			File file = new File(stateFileName);
			file.delete();

			StateManager st = new StateManager();

			int tabCount = chatTabbedPane.getTabCount();
			for (int i = 0; i < tabCount; i++) {
				String tabName = chatTabbedPane.getTitleAt(i);
				st.addTab(tabName);
			}
			st.output(new FileOutputStream(new File(stateFileName)));
		} catch (ParserConfigurationException e1) {
			// e1.printStackTrace();
			JOptionPane.showMessageDialog(mainFrame,
					"Can't save tabs state to file.\n Serious configuration error in XML parser.\n"
							+ e1, "Error", JOptionPane.ERROR_MESSAGE);
		} catch (SAXException e1) {
			// e1.printStackTrace();
			JOptionPane.showMessageDialog(mainFrame,
					"Can't save tabs state to file.\n Parse XML error.\n" + e1,
					"Error", JOptionPane.ERROR_MESSAGE);
		} catch (TransformerException e) {
			// e.printStackTrace();
			JOptionPane.showMessageDialog(mainFrame,
					"Can't save tabs state to file " + stateFileName
							+ ".\n Write XML in file error.\n" + e, "Error",
					JOptionPane.ERROR_MESSAGE);
		} catch (FileNotFoundException e) {
			// e.printStackTrace();
			JOptionPane.showMessageDialog(mainFrame, "Can't find XML file "
					+ stateFileName + " to save tabs state.\n" + e, "Error",
					JOptionPane.ERROR_MESSAGE);
		} catch (IOException e) {
			// e.printStackTrace();
			JOptionPane.showMessageDialog(mainFrame, "Save tabs to xml file "
					+ stateFileName + " io Error.\n" + e, "Error",
					JOptionPane.ERROR_MESSAGE);
		}

		File contactFile = new File(contactListFileName);
		contactFile.delete();

		try {
			popupDataStore.getContactList().putContactsToXMLFile(
					contactListFileName);
		} catch (IOException e) {
			// e.printStackTrace();
			JOptionPane.showMessageDialog(mainFrame,
					"Save contacts to xml file " + contactListFileName
							+ " io Error.\n" + e, "Error",
					JOptionPane.ERROR_MESSAGE);
		} catch (TransformerException e) {
			// e.printStackTrace();
			JOptionPane.showMessageDialog(mainFrame,
					"Can't save contacts to file " + contactListFileName
							+ ".\n Write XML in file error.\n" + e, "Error",
					JOptionPane.ERROR_MESSAGE);
		}
	}

	public void saveSettings() {
		File file = new File(settingsFileName);
		file.delete();
		try {
			settingsManager
					.saveSettings(new FileOutputStream(settingsFileName));
		} catch (FileNotFoundException e) {
			// e.printStackTrace();
			JOptionPane.showMessageDialog(mainFrame, "Can't find XML file "
					+ settingsFileName + " for saving popup settings.\n" + e,
					"Error", JOptionPane.ERROR_MESSAGE);
		} catch (IOException e) {
			// e.printStackTrace();
			JOptionPane.showMessageDialog(mainFrame,
					"Save popup settings to xml file " + settingsFileName
							+ " io Error.\n" + e, "Error",
					JOptionPane.ERROR_MESSAGE);
		} catch (TransformerException e) {
			// e.printStackTrace();
			JOptionPane.showMessageDialog(mainFrame,
					"Can't save popup settings to xml file " + settingsFileName
							+ ".\n Write XML in file error.\n" + e, "Error",
					JOptionPane.ERROR_MESSAGE);
		}
	}

	public void readWindowState() {
		try {
			File contactFile = new File(contactListFileName);
			if (contactFile.exists()) {
				popupDataStore.getContactList().getContactsFromXMLFile(
						contactFile);
				for (Contact contact : popupDataStore.getContactList()) {
					addContactToContactList(contact);
				}
			}
		} catch (ParserConfigurationException e1) {
			// e1.printStackTrace();
			JOptionPane.showMessageDialog(mainFrame,
					"Can't parse contact list xml file " + contactListFileName
							+ ". Serious configuration error in XML parser.\n"
							+ e1, "Error", JOptionPane.ERROR_MESSAGE);
		} catch (SAXException e1) {
			// e1.printStackTrace();
			JOptionPane.showMessageDialog(mainFrame,
					"Can't parse contact list xml file " + contactListFileName
							+ ". Serious configuration error in XML parser.\n"
							+ e1, "Error", JOptionPane.ERROR_MESSAGE);
		} catch (FileNotFoundException e) {
			// e.printStackTrace();
			JOptionPane.showMessageDialog(mainFrame, "Can't find XML file "
					+ contactListFileName + " for readinf contact list.\n" + e,
					"Error", JOptionPane.ERROR_MESSAGE);
		} catch (IOException e) {
			// e.printStackTrace();
			JOptionPane.showMessageDialog(mainFrame,
					"IO Error. Can't read contact list file "
							+ contactListFileName + "\n" + e, "Error",
					JOptionPane.ERROR_MESSAGE);
		} catch (ParseException e) {
			// e.printStackTrace();
			JOptionPane.showMessageDialog(mainFrame,
					"Can't parse contact list xml file " + contactListFileName
							+ ". Parse XML error.\n" + e, "Error",
					JOptionPane.ERROR_MESSAGE);
		}

		File stateFile = new File(stateFileName);

		if (stateFile.exists()) {

			StateManager st;
			try {
				st = new StateManager(stateFile);
				ArrayList<Tab> tabs = st.getTabs();
				for (Tab tab : tabs) {
					createNewTab(tab.getName(), noNewMSGIcon);					
				}
			} catch (ParserConfigurationException e) {
				// e.printStackTrace();
				JOptionPane
						.showMessageDialog(
								mainFrame,
								"Can't parse window state xml file "
										+ stateFileName
										+ ". Serious configuration error in XML parser.\n"
										+ e, "Error", JOptionPane.ERROR_MESSAGE);
			} catch (SAXException e) {
				// e.printStackTrace();
				JOptionPane
						.showMessageDialog(
								mainFrame,
								"Can't parse window state xml file "
										+ stateFileName
										+ ". Serious configuration error in XML parser.\n"
										+ e, "Error", JOptionPane.ERROR_MESSAGE);
			} catch (IOException e) {
				// e.printStackTrace();
				JOptionPane.showMessageDialog(mainFrame,
						"IO Error. Can't read state window xml file "
								+ stateFileName + "\n" + e, "Error",
						JOptionPane.ERROR_MESSAGE);
			}
		}
		
		if (contactListModel.size() == 0) {
			Contact generalGroup = new Contact(GENERAL_TAB_NAME);
			addContactToContactList(generalGroup);
		}
		if (chatTabbedPane.getTabCount() == 0) {
			createNewTab(GENERAL_TAB_NAME, noNewMSGIcon);
		}
	}

	private void readSettings() {
		File settingsFile = new File(settingsFileName);
		try {
			if (settingsFile.exists() && settingsFile.length() > 0) {
				settingsManager = new SettingsManager(settingsFile);
			} else {
				settingsManager = new SettingsManager();
			}
		} catch (ParserConfigurationException e1) {
			// e1.printStackTrace();
			JOptionPane.showMessageDialog(mainFrame,
					"Can't parse settings xml file " + settingsFileName
							+ ". Serious configuration error in XML parser.\n"
							+ e1, "Error", JOptionPane.ERROR_MESSAGE);
		} catch (SAXException e1) {
			// e1.printStackTrace();
			JOptionPane.showMessageDialog(mainFrame,
					"Can't parse settings xml file " + settingsFileName
							+ ". Serious configuration error in XML parser.\n"
							+ e1, "Error", JOptionPane.ERROR_MESSAGE);
		} catch (FileNotFoundException e) {
			// e.printStackTrace();
			JOptionPane.showMessageDialog(mainFrame, "Can't find XML file "
					+ settingsFileName + " for readinf settings.\n" + e,
					"Error", JOptionPane.ERROR_MESSAGE);
		} catch (IOException e) {
			// e.printStackTrace();
			JOptionPane.showMessageDialog(mainFrame,
					"IO Error. Can't read settings file " + contactListFileName
							+ "\n" + e, "Error", JOptionPane.ERROR_MESSAGE);
		}
	}

	private class MainFrameListener extends WindowAdapter {

		private MainWindow mainFrame;
		
		public MainFrameListener(MainWindow mainFrame) {
			this.mainFrame = mainFrame;
		}
		
		public void windowClosing(WindowEvent e) {
			saveWindowState();
			saveSettings();
		}

		public void windowOpened(WindowEvent e) {
			readWindowState();
		}
		
		public void windowActivated(WindowEvent arg0) { 
			super.windowActivated(arg0);
			if (mainFrame.getMainFrame().getIconImage().equals(getNewMSGIcon().getImage())) {
				int tabIndex = getChatTabbedPane().getSelectedIndex();
				if (getChatTabbedPane().getIconAt(tabIndex).equals(getNewMSGIcon())) {
					getChatTabbedPane().setIconAt(tabIndex, getNoNewMSGIcon());
					notificationCount--;
					if (notificationCount == 0) {
						mainFrame.getMainFrame().setIconImage(noMSGMainIcon.getImage());
					}
				}
			}
		}		
	}

	public String preparingTextMsg(String dirtyMSG) {
		String clearMSG = new String(dirtyMSG);
		for (Entry<String, String> escapeXml : escapeXmls.entrySet()) {
			clearMSG = clearMSG.replaceAll(escapeXml.getKey(), escapeXml
					.getValue());
		}
		return clearMSG;
	}
		
	public String addTagsToMsg(String msg, String sender, Date time) {
		if (time == null) {
			time = new Date();
		}
		StringBuffer sb = new StringBuffer();
		String timeString = DateFormat.getInstance().format(time,sb,new FieldPosition(DateFormat.AM_PM_FIELD)).toString();
		String newString = statrSMGPattern 
						   + "<table cellpadding=0 cellspacing=0 width=100%>"
						   	    + "<tr bgcolor=" + msgHeaderColor + ">"
						   			+ "<td>" 
						   				+ "<b><a href="+ sender + ">" + sender + "</a></b>" 
						   			+ "</td>"
						   			+ "<td align=right>" 
						   				+ " <b>" + timeString + "</b>" 
					   				+ "</td>"
						   		+ "</tr>" 
						   		+ "<tr>" 
						   			+ "<td colspan = 2>" 
						   				+ msg 
						   			+ "</td>"
						   		+ "</tr>" 
						   	+ "</table>";		
		return newString;
	}

	public MyPopupListener getMyPopupListener() {
		return new MyPopupListener();
	}
	
	private void fitMsgList(String tabTitle) {
		StringBuilder msgList = popupDataStore.getMsgList(tabTitle);
		int start = msgList.indexOf(statrSMGPattern, msgList.indexOf(statrSMGPattern) + 1);
		msgList.replace(0, start, "");
	}	

	public void addTabToTabbedPane(String tabName , Icon icon) {
		if (chatTabbedPane.indexOfTab(tabName) < 0) {
			createNewTab(tabName, icon);
		}
		chatTabbedPane.setSelectedIndex(chatTabbedPane.indexOfTab(tabName));
	}
	
	public void removeTab(String tabName) {
		chatTabbedPane.remove(chatTabbedPane.indexOfTab(tabName));
		scrollPanes.remove(tabName);
	}

	public void createNewTab(String title, Icon icon) {
		title = title.toUpperCase();
		JEditorPane chatTextPane;
		if (chatTextList.get(title) == null) {
			chatTextPane = new JEditorPane();
			chatTextPane.setEditable(false);
			chatTextPane.setEditorKit(new HTMLEditorKit());
			chatTextPane.setText(startTags + finishTags);
			chatTextPane.addHyperlinkListener(mainHyperlinkClickEvent);

			chatTextList.put(title, chatTextPane);
			StringBuilder msgList = new StringBuilder();
			popupDataStore.addMsgList(title, msgList);
			String historyFileName = MessageHistory.HISTORY_DIRECTORY + "/"
					+ title + "." + MessageHistory.HISTORY_FILE_EXT;
			try {
				XMLMessageHistory history;
				history = new XMLMessageHistory(title);
				history.getHistoryFromXMLFile();
				popupDataStore.addHistory(title, history);
				if (popupDataStore.getContactList().getContact(title) != null) {
					popupDataStore.getContactList().getContact(title)
							.setHistory(history);					
				}				
				
				ArrayList<Message> last = history.getLastMessages(visibleMsgsCount);

				msgList = popupDataStore.getMsgList(title);
				for (Message msg: last) {						
					String newMSGText = preparingTextMsg(msg.getText());						
					msgList.append(addTagsToMsg(newMSGText, msg.getSender(), msg.getTime()));						
				}
				chatTextPane.setText(startTags + msgList + finishTags);
				chatTextPane.setSelectionStart(
						chatTextPane.getText().length() - 1);
				visibleCount.put(title, last.size()); 
			} catch (ParserConfigurationException e1) {
				// e1.printStackTrace();
				JOptionPane
						.showMessageDialog(
								mainFrame,
								"Can't parse history xml file "
										+ historyFileName
										+ ". Serious configuration error in XML parser.\n"
										+ e1, "Error",
								JOptionPane.ERROR_MESSAGE);
			} catch (FileNotFoundException e) {
				// e.printStackTrace();
				JOptionPane.showMessageDialog(mainFrame, "Can't find XML file "
						+ historyFileName + " for reading history.\n" + e,
						"Error", JOptionPane.ERROR_MESSAGE);
			} catch (IOException e) {
				// e.printStackTrace();
				JOptionPane.showMessageDialog(mainFrame,
						"IO Error. Can't read history file " + historyFileName
								+ "\n" + e, "Error", JOptionPane.ERROR_MESSAGE);
			} catch (SAXException e) {
				JOptionPane.showMessageDialog(mainFrame,
						"Can't parse history XML file:\n" + e, "Error", JOptionPane.ERROR_MESSAGE);
				// e.printStackTrace();
			} catch (ParseException e) {
				JOptionPane.showMessageDialog(mainFrame,
						"Can't parse history XML file:\n" + e, "Error", JOptionPane.ERROR_MESSAGE);
				// e.printStackTrace();
			}
		} else {
			chatTextPane = chatTextList.get(title);
		}

		int newTabIndex = chatTabbedPane.indexOfTab(title);
		if (newTabIndex < 0) {
			JScrollPane chatScrollpane = new JScrollPane(chatTextPane);
			chatScrollpane
					.setHorizontalScrollBarPolicy(JScrollPane.HORIZONTAL_SCROLLBAR_AS_NEEDED);
			chatScrollpane
					.setVerticalScrollBarPolicy(JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED);

			chatTabbedPane.addTab(title, icon, chatScrollpane);
			scrollPanes.put(title, chatScrollpane);
		} else {
			chatTabbedPane.setSelectedIndex(newTabIndex);
		}
	}

	private class MyPopupListener implements PopupListener {		

		public void getPopupMSG(PopupMSG msg) {
			TextPopupMSG textPopupMSG = (TextPopupMSG)msg;
			String newMSGText = preparingTextMsg(textPopupMSG.getText().toString());
			StringBuilder msgList;
			String tabTitle;
			XMLMessageHistory history;
			String sender = textPopupMSG.getSender().toUpperCase();

			if (!checkMessage(textPopupMSG)) {
				return;
			}

			if (myName.equalsIgnoreCase(textPopupMSG.getReceiver())) {
				if (getChatTabbedPane().indexOfTab(sender) < 0) {
					createNewTab(sender, getNewMSGIcon());
					notificationCount++;
				}
				msgList = getPopupDataStore().getMsgList(sender);
				tabTitle = sender;
				history = getPopupDataStore().getHistory(sender);
			} else if (myName.equalsIgnoreCase(textPopupMSG.getSender()) && !GENERAL_TAB_NAME.equalsIgnoreCase(textPopupMSG.getReceiver())) {
				return;				
			} else {
				if (getChatTabbedPane().indexOfTab(GENERAL_TAB_NAME) < 0) {
					createNewTab(GENERAL_TAB_NAME, getNewMSGIcon());
					notificationCount++;
				}
				msgList = getPopupDataStore().getMsgList(GENERAL_TAB_NAME);
				tabTitle = GENERAL_TAB_NAME;
				history = getPopupDataStore().getHistory(GENERAL_TAB_NAME);
			}			

			Date time = new Date();
			history.addMessage(sender, textPopupMSG.getText().toString(), Message
					.getDateFormat().format(time));
			
			Integer msgsCount = visibleCount.get(tabTitle);
			if (msgsCount >= visibleMsgsCount) {
				fitMsgList(tabTitle);
			} else {
				msgsCount++;
			}
										
			int tabIndex = getChatTabbedPane().indexOfTab(tabTitle);	
			
			msgList.append(addTagsToMsg(newMSGText, sender, time));	
			chatTextList.get(tabTitle).setText(getStartTags() + msgList + getFinishTags());	
			
			chatTextList.get(tabTitle).setSelectionStart(
					chatTextList.get(tabTitle).getText().length() - 1);			
						
			if (!mainFrame.isFocused() || getChatTabbedPane().getSelectedIndex() != tabIndex) {
				mainFrame.setIconImage(getMSGMainIcon.getImage());
				if (getChatTabbedPane().getIconAt(tabIndex).equals(getNoNewMSGIcon())) {
					getChatTabbedPane().setIconAt(tabIndex, getNewMSGIcon());
					notificationCount++;
				}
			}
		}		
	}
	
	private class TabMouseListener extends MouseAdapter {

		public void mouseClicked(MouseEvent e) {
			switch (e.getButton()) {
			
			case MouseEvent.BUTTON1:
				if (e.getClickCount() == 2) {
					if (getChatTabbedPane().getTabCount() > 0) {
						String removeTabName = getChatTabbedPane()
								.getTitleAt(getChatTabbedPane().getSelectedIndex());
						removeTab(removeTabName);
					}
				}
				break;
			case MouseEvent.BUTTON3:
				String nameOfActiveTab = getChatTabbedPane().getTitleAt(getChatTabbedPane().getSelectedIndex()).toUpperCase();
				Contact newContact = getPopupDataStore().getContactList().getContact(nameOfActiveTab);
				if (newContact != null) {
					((JMenuItem)tabMenu.getSubElements()[TAB_ADD_CONTACT_INDEX]).setEnabled(false);
				} else {
					((JMenuItem)tabMenu.getSubElements()[TAB_ADD_CONTACT_INDEX]).setEnabled(true);
				}
				tabMenu.show(e.getComponent(), e.getX(), e.getY());				
			break;
			}			
		}
	}

	public void addContactToContactList(Contact contact) {
		Contact newContact = popupDataStore.getContactList().getContact(contact.getUID());
		if (newContact != null && !contactListModel.contains(newContact) || newContact == null) {
			if (newContact == null) {
				popupDataStore.getContactList().addContact(contact);
				newContact = contact;
			}
			
			try {
				XMLMessageHistory history = new XMLMessageHistory(contact.getUID());
				newContact.setHistory(history);
			} catch (ParserConfigurationException e) {
				JOptionPane.showMessageDialog(mainFrame,
						"Can't parse xml history for\"" + contact.getUID() +  "\".\n" + e,
						"Error", JOptionPane.ERROR_MESSAGE);
				// e.printStackTrace();
			} catch (IOException e) {
				JOptionPane.showMessageDialog(mainFrame,
						"Can't read history for\"" + contact.getUID() +  "\".\n" + e,
						"Error", JOptionPane.ERROR_MESSAGE);
				// e.printStackTrace();
			}
			
			int counter = 0;
			for (; counter < contactListModel.size(); counter++) {
				if (((Contact)contactListModel.getElementAt(counter)).compareTo(newContact) > 0) {
					break;
				}
			}
			
			contactListModel.insertElementAt(newContact, counter);
		}
	}

	public void removeContactFromContactList(Contact contact) {
		if (contactListModel.contains(contact)) {
			contactListModel.removeElement(contact);
			popupDataStore.getContactList().removeContact(contact.getUID());
		}
	}
	
	public boolean checkMessage(TextPopupMSG msg) {
		Message newMessage = null;
		try {
			newMessage = new Message(msg.getSender(), msg.getText().toString(),
					Message.getDateFormat().format(new Date()));

			for (Iterator<MessageFilter> filterIterator = popupDataStore
					.getMessageFilterIterator(); filterIterator.hasNext();) {
				MessageFilter filter = filterIterator.next();
				if (!filter.isValidMessage(newMessage)) {
					return false;
				}
			}
		} catch (ParseException e) {
			// e.printStackTrace();
			JOptionPane.showMessageDialog(mainFrame,
					"Wrong date format for birthday in contact.\n" + e,
					"Error", JOptionPane.ERROR_MESSAGE);
			return false;
		}
		return true;
	}
	
	public PopupCatcher chooseEthernetDevice(boolean isNew, boolean automatic) {
		boolean isDeviceSelected = false;
		String curentEthernetDevice = "";
		String nameOfEthernetDevice = settingsManager.getEthernetDevice();
		if (!isNew) {
			curentEthernetDevice = settingsManager.getEthernetDevice();			
		}
		PopupCatcher catcher = null;
		int counter = 0;
		while (!isDeviceSelected) {
			try {					
				if (curentEthernetDevice.equals("")) {
					String[] ethernetDevices = PacketCapture.lookupDevices();
					String[] devicesNames = new String[ethernetDevices.length];
					
					int currentDeviceNumber = 0;
					for (int i = 0; i < ethernetDevices.length; i++) {
						if (currentDeviceNumber == 0 && ethernetDevices[i].contains(nameOfEthernetDevice)) {
							currentDeviceNumber = i;
						}
						devicesNames[i] = ethernetDevices[i].substring(
								ethernetDevices[i].indexOf("\n")+3);							
					}
					if (!automatic) {
						nameOfEthernetDevice = (String) JOptionPane
							.showInputDialog(mainFrame,
									"Choose ethernet device",
									"Device chooser",
									JOptionPane.QUESTION_MESSAGE, null,
									devicesNames, devicesNames[currentDeviceNumber]);
					} else {
						if (counter >= ethernetDevices.length) {
							throw new CaptureDeviceLookupException("All devices can't be opend");
						}
						nameOfEthernetDevice = devicesNames[counter];
						counter++;
					}
					
					if (nameOfEthernetDevice == null) {
						sendAction.setEnabled(false);
						return null;
					}
					for (int i = 0; i < devicesNames.length; i++) {
						if (devicesNames[i].equalsIgnoreCase(nameOfEthernetDevice)) {								
							curentEthernetDevice = ethernetDevices[i].substring(0, ethernetDevices[i].indexOf("\n"));
							settingsManager.setEthernetDevice(curentEthernetDevice);
						}
					}					
				}
				WindowsPopupDaemon daemon = new WindowsPopupDaemon(curentEthernetDevice);
				catcher = new WindowsPopupCatcher(daemon);
				settingsManager.setEthernetDevice(curentEthernetDevice);
				isDeviceSelected = true;
			} catch (CaptureDeviceLookupException e) {
				// e.printStackTrace();
				JOptionPane
						.showMessageDialog(
								mainFrame,
								"There are no ethernet devices to capture messages.",
								"Error", JOptionPane.WARNING_MESSAGE);
				sendAction.setEnabled(false);
				return null;				
			} catch (CaptureDeviceOpenException e) {
				// e.printStackTrace();
				if (!automatic) {
					JOptionPane.showMessageDialog(mainFrame,
						"Can't open selected ethernet device \""
								+ nameOfEthernetDevice + "\".", "Error",
						JOptionPane.ERROR_MESSAGE);
				}
				curentEthernetDevice = "";
				nameOfEthernetDevice = "";
			}
		}
		sendAction.setEnabled(isDeviceSelected);
		return catcher;
	}

	private class RemoveContactAction implements ActionListener {

		public void actionPerformed(ActionEvent arg0) {
			Contact selectedContactName = (Contact) contactList.getSelectedValue();
			removeContactFromContactList(selectedContactName);
		}
	}

	private class ViewContactInfoAction extends AbstractAction {
		private static final long serialVersionUID = -8104482819686873856L;
		
		private static final String DESCRIPTION = "View contact info";
		
		public static final int CONTACT_LIST = 0;
		public static final int TOOLBAR = 1;
		
		private MainWindow mainFrame;
		private int sourse;
		
		public ViewContactInfoAction(MainWindow mainFrame, int sourse) {
			this.mainFrame = mainFrame;
			putValue(AbstractAction.SMALL_ICON, viewContactInfoIcon);
			putValue(AbstractAction.SHORT_DESCRIPTION, DESCRIPTION);
			this.sourse = sourse;
		}
		
		public void actionPerformed(ActionEvent e) {
			String selectedContactTitle = "";
			switch (sourse) {
			case CONTACT_LIST:
				selectedContactTitle = ((Contact)contactList.getSelectedValue()).getUID();
				break;
			case TOOLBAR:
				selectedContactTitle = getChatTabbedPane().getTitleAt(getChatTabbedPane().getSelectedIndex());
				if (getPopupDataStore().getContactList().getContact(selectedContactTitle) == null) {
					JOptionPane.showMessageDialog(getMainFrame(), "There is no user \"" + selectedContactTitle + "\" in contact list.\n"
																+ "If you want view contact information you should add this user to contact list.", "Warning", JOptionPane.WARNING_MESSAGE);
					return;
				}
				break;
			}			
			ContactWindow contactWindow = new ContactWindow(getPopupDataStore()
					.getContactList().getContact(selectedContactTitle), mainFrame);
			contactWindow.pack();
			contactWindow.setVisible(true);
		}
	}

	private class PingContactAction extends AbstractAction {

		private static final long serialVersionUID = -8187879496792518022L;
		
		private static final String DESCRIPTION = "Ping computer";
		
		public static final int CONTACT_LIST = 0;
		public static final int TOOLBAR = 1;
		
		private int source;
		
		public PingContactAction(int source) {
			putValue(AbstractAction.SMALL_ICON, getPingIcon());
			putValue(AbstractAction.SHORT_DESCRIPTION, DESCRIPTION);
			this.source = source;			
		}
		
		public void actionPerformed(ActionEvent e) {
			String selectedContactName = "";
			
			switch (source) {
			case CONTACT_LIST:
				selectedContactName = ((Contact)contactList.getSelectedValue()).getUID();
				break;
			case TOOLBAR:
				selectedContactName = getChatTabbedPane().getTitleAt(getChatTabbedPane().getSelectedIndex());				
			}
			
			InetAddress address = null;
			try {
				address = InetAddress.getByName(selectedContactName);
			} catch (UnknownHostException e1) {}
			
			if (address != null) {
				JOptionPane.showMessageDialog(getMainFrame(), "Computer \"" + selectedContactName + "\" is ONLINE now.\n" 
						+ "IP: " + address.getHostAddress(), "ONLINE", JOptionPane.INFORMATION_MESSAGE, getUserOnlineIcon());
			} else {
				JOptionPane.showMessageDialog(getMainFrame(), "Computer \"" + selectedContactName + "\" is OFFLINE now.", "OFFLINE", JOptionPane.INFORMATION_MESSAGE, getUserOfflineIcon());
			}
		}
	}

	private class CreateTabAction implements ActionListener {

		public void actionPerformed(ActionEvent e) {
			String selectedContactName = ((Contact) contactList
					.getSelectedValue()).getUID();
			addTabToTabbedPane(selectedContactName, getNoNewMSGIcon());
		}
	}

	private class ContactListMouseListener extends MouseAdapter {
		public void mouseClicked(MouseEvent e) {
			switch (e.getButton()) {
			case MouseEvent.BUTTON3:
				int selectedIndex = contactList.locationToIndex(e.getPoint());
				contactList.setSelectedIndex(selectedIndex);
				contactListMenu.show(e.getComponent(), e.getX(), e.getY());
				break;
			case MouseEvent.BUTTON1:
				if (e.getClickCount() == 2) {
					String receiver = ((Contact) contactList.getSelectedValue())
							.getUID().toUpperCase();
					addTabToTabbedPane(receiver, getNoNewMSGIcon());
				}					
				break;
			}
		}
	}

	private class TabChangeListener implements ChangeListener {

		public void stateChanged(ChangeEvent arg0) {
			int tabIndex = getChatTabbedPane().getSelectedIndex();
			if (tabIndex > -1) {
				if (getChatTabbedPane().getIconAt(tabIndex).equals(getNewMSGIcon())) {				
					getChatTabbedPane().setIconAt(tabIndex, getNoNewMSGIcon());
					notificationCount--;
					if (notificationCount == 0) {
						mainFrame.setIconImage(noMSGMainIcon.getImage());
					}
				}				
				mainFrame.setTitle(chatTabbedPane.getTitleAt(tabIndex));
			} else {
				mainFrame.setTitle(DEFAULT_TITLE);
			}
		}		
	}	
	
	private class SendAction extends AbstractAction {
		private static final long serialVersionUID = 1L;
//		private static final String SEND_BUTTON_NAME = "Send";
		private static final String DESCRIPTION = "Send Message";
		
		public SendAction() {
			putValue(AbstractAction.SMALL_ICON, sendIcon);
			putValue(AbstractAction.SHORT_DESCRIPTION, DESCRIPTION);
		}

		public void actionPerformed(ActionEvent e) {			
			String msgText = inputTextArea.getText();
			String receiver = getChatTabbedPane().getTitleAt(getChatTabbedPane()
					.getSelectedIndex());
			
			try {
				if (!getJPopup().sendMessage(myName, receiver, msgText.getBytes(PopupSMBMessage.getOutEncoding()))) {
					JOptionPane.showMessageDialog(getMainFrame(),
							"Sending message to " + receiver
							+ " faild.\n" 
							+ "Can't find " + receiver + " in network.", "Error",
							JOptionPane.ERROR_MESSAGE);
				} else { 
					if (!receiver.equals(GENERAL_TAB_NAME)) {
						String newMsgText = preparingTextMsg(msgText);
						
						Integer msgsCount = visibleCount.get(receiver);
						if (msgsCount >= visibleMsgsCount) {
							fitMsgList(receiver);
						} else {
							msgsCount++;
						}
						
						StringBuilder msgList = getPopupDataStore().getMsgList(receiver);

						msgList.append(addTagsToMsg(newMsgText, myName, null));
						chatTextList.get(receiver).setText(
								getStartTags() + msgList + getFinishTags());
						XMLMessageHistory history = getPopupDataStore().getHistory(receiver);
						history.addMessage(myName, msgText, Message.getDateFormat()
								.format(new Date()));
					}
					inputTextArea.setText("");
				}
			} catch (UnsupportedEncodingException uee) {
				System.err.println(uee);
			//	uee.printStackTrace();
				JOptionPane.showMessageDialog(null,
						"Received message has unsupported encoding.\n" + uee, "Warning",
						JOptionPane.WARNING_MESSAGE);
			}
		}
	}

	public void setMainMenu(JMenuBar mainMenu) {
		this.mainMenu = mainMenu;
	}

	public JMenuBar getMainMenu() {
		return mainMenu;
	}

	public JMenu getFiltersMenu() {
		return filtersMenu;
	}

	public void setMainFrame(JFrame mainFrame) {
		this.mainFrame = mainFrame;
	}

	public JFrame getMainFrame() {
		return mainFrame;
	}

	public void setVisibleMsgsCount(int visibleMsgsCount) {
		this.visibleMsgsCount = visibleMsgsCount;
	}

	public int getVisibleMsgsCount() {
		return visibleMsgsCount;
	}

	public static void setEscapeSequences(LinkedHashMap<String, String> escapeXmls) {
		MainWindow.escapeXmls = escapeXmls;
	}

	public static LinkedHashMap<String, String> getEscapeSequences() {
		return escapeXmls;
	}	
	
	public void setStartTags(String startTags) {
		this.startTags = startTags;
	}

	public String getStartTags() {
		return startTags;
	}

	public void setFinishTags(String finishTags) {
		this.finishTags = finishTags;
	}

	public String getFinishTags() {
		return finishTags;
	}

	public void setMainHyperlinkClickEvent(ChatHyperlinkClickListener mainHyperlinkClickEvent) {
		this.mainHyperlinkClickEvent = mainHyperlinkClickEvent;
	}

	public ChatHyperlinkClickListener getMainHyperlinkClickEvent() {
		return mainHyperlinkClickEvent;
	}

	public void setMsgHeaderColor(String msgHeaderColor) {
		this.msgHeaderColor = msgHeaderColor;
	}

	public String getMsgHeaderColor() {
		return msgHeaderColor;
	}

	public void setUserOnlineIcon(ImageIcon userOnlineIcon) {
		this.userOnlineIcon = userOnlineIcon;
	}

	public ImageIcon getUserOnlineIcon() {
		return userOnlineIcon;
	}

	public void setUserOfflineIcon(ImageIcon userOfflineIcon) {
		this.userOfflineIcon = userOfflineIcon;
	}

	public ImageIcon getUserOfflineIcon() {
		return userOfflineIcon;
	}

	public void setHistoryIcon(ImageIcon historyIcon) {
		this.historyIcon = historyIcon;
	}

	public ImageIcon getHistoryIcon() {
		return historyIcon;
	}

	public void setPingIcon(ImageIcon pingIcon) {
		this.pingIcon = pingIcon;
	}

	public ImageIcon getPingIcon() {
		return pingIcon;
	}

	public void setChatTabbedPane(JTabbedPane chatTabbedPane) {
		this.chatTabbedPane = chatTabbedPane;
	}

	public JTabbedPane getChatTabbedPane() {
		return chatTabbedPane;
	}

	public void setNoNewMSGIcon(ImageIcon noNewMSGIcon) {
		this.noNewMSGIcon = noNewMSGIcon;
	}

	public ImageIcon getNoNewMSGIcon() {
		return noNewMSGIcon;
	}

	public void setNewMSGIcon(ImageIcon newMSGIcon) {
		this.newMSGIcon = newMSGIcon;
	}

	public ImageIcon getNewMSGIcon() {
		return newMSGIcon;
	}

	public void setJPopup(JPopupEngine jPopup) {
		this.jPopup = jPopup;
	}

	public JPopupEngine getJPopup() {
		return jPopup;
	}

	public void setDeviceIcon(ImageIcon deviceIcon) {
		this.deviceIcon = deviceIcon;
	}

	public ImageIcon getDeviceIcon() {
		return deviceIcon;
	}

	public void setAboutIcon(ImageIcon aboutIcon) {
		this.aboutIcon = aboutIcon;
	}

	public ImageIcon getAboutIcon() {
		return aboutIcon;
	}

	public void setAddContactString(JTextField addContactString) {
		this.addContactString = addContactString;
	}

	public JTextField getAddContactString() {
		return addContactString;
	}
	
	public void setSendIcon(ImageIcon sendIcon) {
		this.sendIcon = sendIcon;
	}

	public ImageIcon getSendIcon() {
		return sendIcon;
	}

	public void setSendAction(SendAction sendAction) {
		this.sendAction = sendAction;
	}

	public SendAction getSendAction() {
		return sendAction;
	}

	public void setPopupDataStore(PopupDataStore popupDataStore) {
		this.popupDataStore = popupDataStore;
	}

	public PopupDataStore getPopupDataStore() {
		return popupDataStore;
	}	
}
