package jpopup.gui.actions;

import java.awt.event.ActionEvent;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.text.ParseException;

import javax.swing.AbstractAction;
import javax.swing.Action;
import javax.swing.JOptionPane;

import jpopup.gui.HistoryWindow;
import jpopup.gui.MainWindow;
import jpopup.history.MessageHistory;
import jpopup.history.XMLMessageHistory;

import org.xml.sax.SAXException;

/**
 * This action used to show message history of contact wich displayed in current tab in main window
 * @author Alexey Kalukin
 *
 */
public class ShowHistoryAction extends AbstractAction {

	private static final long serialVersionUID = -7579015796904413717L;

	private static final String SHOW_HYSOTRY_BUTTON_NAME = "History";
	private static final String DESCRIPTION = "History";

	private MainWindow mainWindow;		
	
	public ShowHistoryAction(MainWindow mainWindow) {
		this.mainWindow = mainWindow;
		putValue(Action.NAME, SHOW_HYSOTRY_BUTTON_NAME);
		putValue(Action.SMALL_ICON, mainWindow.getHistoryIcon());
		putValue(AbstractAction.SHORT_DESCRIPTION, DESCRIPTION);
	}

	public void actionPerformed(ActionEvent arg0) {
		String selectedTabTitle = mainWindow.getChatTabbedPane().getTitleAt(mainWindow.getChatTabbedPane()
				.getSelectedIndex());
		XMLMessageHistory history = mainWindow.getPopupDataStore()
				.getHistory(selectedTabTitle);

		String historyFileName = MessageHistory.HISTORY_DIRECTORY + "/"
				+ history.getContactTitle() + "."
				+ MessageHistory.HISTORY_FILE_EXT;
		try {
			history.getHistoryFromXMLFile();
		} catch (SAXException e1) {
			// e1.printStackTrace();
			JOptionPane.showMessageDialog(mainWindow.getMainFrame(),
					"Parse XML error. Can't read history xml file "
							+ historyFileName + ".\n" + e1, "Error",
					JOptionPane.ERROR_MESSAGE);
		} catch (FileNotFoundException e) {
			// e.printStackTrace();
			JOptionPane.showMessageDialog(mainWindow.getMainFrame(),
					"Can't find XML file " + historyFileName
							+ " for reading history.\n" + e, "Error",
					JOptionPane.ERROR_MESSAGE);
		} catch (IOException e) {
			// e.printStackTrace();
			JOptionPane
					.showMessageDialog(mainWindow.getMainFrame(),
							"IO Error with history xml file "
									+ historyFileName + " \n" + e, "Error",
							JOptionPane.ERROR_MESSAGE);
		} catch (ParseException e) {
			JOptionPane.showMessageDialog(mainWindow.getMainFrame(), "Parse XML file "
					+ historyFileName + ".\n" + e, "Error",
					JOptionPane.ERROR_MESSAGE);
			// e.printStackTrace();
		}
		HistoryWindow hw = new HistoryWindow(history, mainWindow);
		hw.pack();
		hw.setVisible(true);
	}

}
