package jpopup.gui.xml;

import java.io.File;
import java.io.IOException;
import java.io.OutputStream;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerException;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.dom.DOMSource;
import javax.xml.transform.stream.StreamResult;

import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.NodeList;
import org.w3c.dom.Text;
import org.xml.sax.ErrorHandler;
import org.xml.sax.SAXException;
import org.xml.sax.SAXParseException;

/**
 * This class used ot store settings of applications, read/write them from/to XML file. 
 * Settings like: current ethernet device;
 * @author Alexey Kalukin
 *
 */
public class SettingsManager {

	private static final String MAIN_CONTAINER = "settings";

	private DocumentBuilderFactory dbf;
	private DocumentBuilder parser;
	private Document document;
	
	private String ethernetDevice;

	public SettingsManager() throws ParserConfigurationException, SAXException,
			IOException {
		dbf = DocumentBuilderFactory.newInstance();
		dbf.setValidating(false);

		parser = dbf.newDocumentBuilder();

		parser.setErrorHandler(new ErrorHandler() {

			public void error(SAXParseException exception) throws SAXException {
				System.err.println("ERROR: " + exception.getMessage());
			}

			public void fatalError(SAXParseException exception)
					throws SAXException {
				System.err.println("FATAL: " + exception.getMessage());
				throw exception;
			}

			public void warning(SAXParseException exception)
					throws SAXException {
				System.err.println("WARNING: " + exception.getMessage());
			}

		});

		document = parser.newDocument();
		document.appendChild(document.createElement(MAIN_CONTAINER));
		getSettingsFromXML();
	}

	public SettingsManager(File xmlFile) throws ParserConfigurationException,
			SAXException, IOException {
		this();
		
		if (xmlFile.exists()) {
			document = parser.parse(xmlFile);
		} else {
			document = parser.newDocument();
			document.appendChild(document.createElement(MAIN_CONTAINER));
		}
		
		getSettingsFromXML();
	}

	private void getSettingsFromXML() {
		
		NodeList ethernets = document.getElementsByTagName("ethernetDevice");
		int ethernetsCount = ethernets.getLength();
		if (ethernetsCount >= 1) {
			Element device = (Element)ethernets.item(0);
			Text deviceName = ((Text) device.getFirstChild());
			if (deviceName != null) {
				ethernetDevice = deviceName.getData().trim();
			} else {
				ethernetDevice = "";
			}
		} else {
			ethernetDevice = "";
		}
	}
	
	public void saveSettings(OutputStream out) throws IOException, TransformerException {
		NodeList ethernets = document.getElementsByTagName("ethernetDevice");
		int ethernetsCount = ethernets.getLength();
		if (ethernetsCount >= 1) {
			Element device = (Element)ethernets.item(0);
			Text deviceName = ((Text) device.getFirstChild());
			if (deviceName != null) {
				deviceName.setData(ethernetDevice);
			}
		} else {
			Element deviceElement = document.createElement("ethernetDevice");
			deviceElement.appendChild(document.createTextNode(ethernetDevice));
			
			document.getElementsByTagName(MAIN_CONTAINER).item(0).appendChild(deviceElement);
		}
		
		output(out);
	}
	
	private void output(OutputStream out) throws IOException,
			TransformerException {
		TransformerFactory factory = TransformerFactory.newInstance();
		Transformer transformer = factory.newTransformer();
		transformer.transform(new DOMSource(document), new StreamResult(out));
	}

	public void setEthernetDevice(String ethernetDevice) {
		this.ethernetDevice = ethernetDevice;
	}

	public String getEthernetDevice() {
		return ethernetDevice;
	}
	
	
}
