package jpopup.gui.xml;

import java.io.File;
import java.io.IOException;
import java.io.OutputStream;
import java.util.ArrayList;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerException;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.dom.DOMSource;
import javax.xml.transform.stream.StreamResult;

import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.NodeList;
import org.w3c.dom.Text;
import org.xml.sax.ErrorHandler;
import org.xml.sax.SAXException;
import org.xml.sax.SAXParseException;

/**
 * This class used to save/read tabs positions in main window when it's closed/opened.
 * @author Alexey Kalukin
 *
 */
public class StateManager {
	
	private static final String MAIN_CONTAINER = "popup-state";
	private static final String ROOT_TAG = "tabs";
	private static final String TAB_TAG = "tab";
	private static final String TAB_NAME_TAG = "tab-name";
	
	private DocumentBuilderFactory dbf;
	private DocumentBuilder parser;
	private Document document;
	
	public StateManager() throws ParserConfigurationException, 
	   							SAXException, 
	   							IOException {
		dbf = DocumentBuilderFactory.newInstance();
		dbf.setValidating(false);

		parser = dbf.newDocumentBuilder();

		parser.setErrorHandler(new ErrorHandler() {

		public void error(SAXParseException exception) throws SAXException {
		System.err.println("ERROR: " + exception.getMessage());				
		}

		public void fatalError(SAXParseException exception) throws SAXException {
		System.err.println("FATAL: " + exception.getMessage());
		throw exception;
		}

		public void warning(SAXParseException exception) throws SAXException {
		System.err.println("WARNING: " + exception.getMessage());				
		}

		});

		document = parser.newDocument();
		document.appendChild(document.createElement(MAIN_CONTAINER));
	}
	
	public StateManager(File xmlFile) throws ParserConfigurationException, 
										   SAXException, 
										   IOException {
		this();
		
		document = parser.parse(xmlFile);
	}
		
	public void addTab(String tabName) {
		Element newNode = document.createElement(TAB_TAG);		
		
		Element nameNode = document.createElement(TAB_NAME_TAG);
		nameNode.appendChild(document.createTextNode(tabName));
		
		newNode.appendChild(nameNode);
		
		NodeList tabsNode = document.getElementsByTagName(ROOT_TAG);
		if (tabsNode.getLength() == 0) {
			document.getElementsByTagName(MAIN_CONTAINER).item(0).appendChild(document.createElement(ROOT_TAG));
			tabsNode = document.getElementsByTagName(ROOT_TAG);
		}
		tabsNode.item(0).appendChild(newNode);
	}
	
	public ArrayList<Tab> getTabs() {
		ArrayList<Tab> data = new ArrayList<Tab>();
		NodeList tabs = document.getElementsByTagName(TAB_TAG);
		int tabsCount = tabs.getLength();
		for (int i = 0; i < tabsCount; i++) {
			Element tab = (Element)tabs.item(i);
			NodeList tabName = tab.getElementsByTagName(TAB_NAME_TAG);
			String name = ((Text)tabName.item(0).getFirstChild()).getData().trim();						
			data.add(new Tab(name));
		}
		
		return data;
	}
	
	
	public void output(OutputStream out) throws IOException, TransformerException {
		TransformerFactory factory = TransformerFactory.newInstance();
	    Transformer transformer = factory.newTransformer();
	    transformer.transform(new DOMSource(document), new StreamResult(out)); 
	}
	
	public static class Tab {
		private String name;		
		
		public Tab(String name) {
			this.name = name;
		}
		
		public void setName(String name) {
			this.name = name;
		}
		public String getName() {
			return name;
		}		
	}
}
