package jpopup.history;

import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.InputStream;

/**
 * Pseudo InputStream used for read pseudo xml(without header and root tag).
 * @author Alexey Kalukin
 *
 */
public class FakeInputStream extends InputStream {
	private static final String START_PART = "<?xml version=\"1.0\" encoding=\"Cp1251\" standalone=\"no\"?>" 
											 +"<history>";
	private static final String FINISH_PART = "</history>";
	
	private int startCounter = 0;
	private int finishCounter = 0;
	private FileInputStream stream;
	
	public FakeInputStream(String fileName) throws FileNotFoundException {
		stream = new FileInputStream(fileName);
	}
	
	public int read() throws IOException {
		int curChar;
		if (startCounter < START_PART.length()) {
			curChar = START_PART.charAt(startCounter);
			startCounter++;
		} else if ((curChar = stream.read()) == -1) {
			if (finishCounter < FINISH_PART.length()) {
				curChar = FINISH_PART.charAt(finishCounter);
				finishCounter++;
			} else {
				curChar = -1;
			}
		}
		return curChar;
	}

}
