package jpopup.plugins;

import java.io.File;
import java.io.FilenameFilter;
import java.io.IOException;
import java.io.PrintWriter;
import java.net.URL;
import java.net.URLClassLoader;
import java.util.Arrays;
import java.util.HashMap;

import jpopup.PopupDataStore;
import jpopup.gui.MainWindow;

/**
 * This class used for manipulate plugins files
 * @author Alexey Kalukin
 *
 */
public class PluginEngine {
	public static final String PLUGINS_DIRECTORY = "plugins";
	public static final String PLUGIN_EXT = "jar";
	public static final String PLUGIN_LOG_FILE = PLUGINS_DIRECTORY + "/plugins.log";
	
	private HashMap<String, PopupPlugin> plugins = new HashMap<String, PopupPlugin>();
	private File pluginDirectory;
	private PopupPluginFilenameFilter filter = new PopupPluginFilenameFilter();
	private PopupDataStore popupData;
	private MainWindow mainWindow;
	private PrintWriter pluginLogger;
	
	public PluginEngine(MainWindow mainWindow, PopupDataStore popupData) throws IOException {
		pluginDirectory = new File(PLUGINS_DIRECTORY);
		if (!pluginDirectory.exists()) {			
			pluginDirectory.mkdir();
		}
		
		this.popupData = popupData;
		this.mainWindow = mainWindow;
		
		pluginLogger = new PrintWriter(PLUGIN_LOG_FILE);
	}
	
	public void getPlugins() throws IOException {
		String[] jars = pluginDirectory.list(filter);
		Arrays.sort(jars);
		
		String urlPluginDir = "file://localhost/" + pluginDirectory.getCanonicalPath();
		
		for (String jarName: jars) {
			URL url = new URL(urlPluginDir + "/" + jarName);
			ClassLoader loader = URLClassLoader.newInstance(new URL[]{url});
			try {
				Class pluginClass = loader.loadClass("PopupPluginImpl");
				PopupPlugin plugin = (PopupPlugin)pluginClass.newInstance();
				plugin.startPlugin(mainWindow, popupData);
				plugins.put(plugin.getName(), plugin);				
			} catch (ClassCastException e) {
				// e.printStackTrace();
				pluginLogger.write(jarName + " - PopupPluginImpl class is not implements interface PopupPlugin.\n" + e);
			} catch (ClassNotFoundException e) {
				// e.printStackTrace();
				pluginLogger.write(jarName + " - can't find PopupPluginImpl class.\n" + e);
			} catch (InstantiationException e) {
				// e.printStackTrace();
				pluginLogger.write(jarName + " - can't create instance of PopupPluginImpl class.\n" + e);
			} catch (IllegalAccessException e) {
				// e.printStackTrace();
				pluginLogger.write(jarName + " - can't run requared methods from class PopupPluginImpl.\n" + e);
			}
			
		}
		
		pluginLogger.close();
	}
	
	private class PopupPluginFilenameFilter implements FilenameFilter {

		public boolean accept(File dir, String name) {
			return name.endsWith(PLUGIN_EXT);
		}
		
	}
}
