package jpopup.popupengine;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.ArrayList;
import java.util.Iterator;

/**
 * Abstract class for implements SMBEngines wich can send/receive smb messages
 * @author Alexey Kalukin
 *
 */
public abstract class JSMBEngine implements ActionListener {
	
	private int udpPort = 138;
	private String workGroupMask = "255.255.255.255";
	private ArrayList<String> workGroups = new ArrayList<String>();
	private ArrayList<PopupListener> listeners = new ArrayList<PopupListener>();

	private PopupCatcher catcher;
	
	public JSMBEngine(PopupCatcher catcher) {		
		workGroups.add("MAT_MEX");
		workGroups.add("*");
		workGroups.add("WORKGROUP");
		workGroups.add("MAT_MEH");
		workGroups.add("MAT-MEX");
		workGroups.add("MAT-MEH");	
		
		this.catcher = catcher;		
		this.catcher.addActionListener(this);
	}
	
	public void startListen() {
		catcher.setActive(true);
		catcher.startCatch();
	}
	
	public void stopListen() {
		catcher.stopCatch();
	}
	
	public abstract boolean sendMessage(String sender, String receiver, byte[] data);
	
	public void addPopupListener(PopupListener listener) {
		listeners.add(listener);
	}
	
	public void removeActionListener(PopupListener listener) {
		listeners.remove(listener);
	}
	
	public void actionPerformed(ActionEvent e) {
	}
	
	protected void addWorkGroupIdentifier(String group) {
		workGroups.add(group);
	}
	
	protected void removeWorkGroupIdentifier(String group) {
		workGroups.remove(group);
	}
	
	protected Iterator<String> workGroupIterator() {
		return workGroups.iterator();
	}
	
	protected Iterator<PopupListener> listenersIterator() {
		return listeners.iterator();
	}

	public void setUdpPort(int udpPort) {
		this.udpPort = udpPort;
	}

	public int getUdpPort() {
		return udpPort;
	}

	public void setWorkGroupMask(String workGroupMask) {
		this.workGroupMask = workGroupMask;
	}

	public String getWorkGroupMask() {
		return workGroupMask;
	}	
}
