package jpopup.popupengine;

import java.io.IOException;
import java.net.DatagramPacket;
import java.net.DatagramSocket;
import java.net.SocketException;

import javax.swing.JOptionPane;

import jpopup.popupengine.text.TextPopupMSG;

public class LinuxPopupCatcher extends PopupCatcher {
	private static final int DEFAULT_POPUP_PORT = 138;

	private static final int BUFFER_SIZE = 2048;

	private static final String POPUP_MSG_HEADER = "MAILSLOT\\messngr";

	private int popupPort = DEFAULT_POPUP_PORT;

	private DatagramPacket receivePacket;

	private DatagramSocket socket;

	private byte[] buffer = new byte[BUFFER_SIZE];

	private boolean active = true;

	public LinuxPopupCatcher() throws SocketException {
		receivePacket = new DatagramPacket(buffer, buffer.length);
		socket = new DatagramSocket(popupPort);
	}

	private static boolean isPopupPacket(DatagramPacket packet) {
		String msg = new String(packet.getData(), 0, packet.getLength());
		return msg.contains(POPUP_MSG_HEADER);
	}

	protected void startCatch() {
		
		while (active) {
			try {
				socket.receive(receivePacket);
				if (isPopupPacket(receivePacket)) {
					PopupSMBMessage popupSMBMsg = new PopupSMBMessage(
							receivePacket.getData());
					String text = popupSMBMsg.getText();
					String sender = popupSMBMsg.getSender().toUpperCase();
					String receiver = popupSMBMsg.getReceiver().toUpperCase();
					byte[] senderIP = popupSMBMsg.getSenderIP();
					TextPopupMSG popupMSG = new TextPopupMSG(sender, receiver, text, senderIP);
					newPopupMsg(popupMSG);
				}
				receivePacket.setLength(buffer.length);
			} catch (IOException ioe) {
				System.err.println(ioe);
			//	ioe.printStackTrace();
				JOptionPane.showMessageDialog(null,
											  "IO Error: can't receive UDP packet.\n"+ ioe, "Error", JOptionPane.ERROR_MESSAGE);
			}
		}
	}

	@Override
	public void stopCatch() {
		// TODO Auto-generated method stub
		
	}
}
