package jpopup.popupengine;

import java.io.UnsupportedEncodingException;
import java.net.InetAddress;
import java.net.UnknownHostException;
import java.util.regex.Pattern;

import javax.swing.JOptionPane;

public class PopupSMBMessage extends SMBMessage{
	public final static byte[] DEFAULT_POPUP_PACKET = new byte[]{17,14,-127,39,0,0,0,0,0,-118,0,-89,0,0,32,70,68,69,70,69,79,69,69,69,70,70,67,67,65,67,65,67,65,67,65,67,65,67,65,67,65,67,65,67,65,65,65,0,32,70,67,69,70,69,68,69,70,69,74,70,71,69,70,70,67,67,65,67,65,67,65,67,65,67,65,67,65,67,65,65,65,0,-1,83,77,66,37,0,0,0,0,24,4,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,-1,-2,0,0,0,0,17,0,0,21,0,2,0,0,0,0,0,2,0,0,0,0,0,0,0,0,0,88,0,21,0,88,0,3,0,1,0,0,0,2,0,40,0,92,77,65,73,76,83,76,79,84,92,109,101,115,115,110,103,114,0,0,115,101,110,100,101,114,0,114,101,99,101,105,118,101,114,0,116,101,120,116,0};
	public final static Pattern SEPARATOR_PATTERN = Pattern.compile("\0");
	public final static int SENDER_FIELD = 0;
	public final static int RECEIVER_FIELD = 1;
	public final static int TEXT_FIELD = 2;
	
	private final static int BIT_IN_BYTE = 256;
	
	private static String outEncoding = "Cp866";
	private static String inEncoding = "Cp866";
	
	public PopupSMBMessage(String sender, String receiver, String text) throws UnknownHostException, UnsupportedEncodingException {
		super(DEFAULT_POPUP_PACKET);
		
		byte[] ip = InetAddress.getLocalHost().getAddress();		
		getNetBIOSDatagramService().setSourceIP(ip);
		
		setSender(sender);
		setReceiver(receiver);
		setText(sender, receiver, text);		
	}
	
	public PopupSMBMessage() {		
	}
	
	public PopupSMBMessage(byte[] data) {
		super(data);
	}
	
	public void setSender(String sender) {
		byte[] srcName = SMBMessage.nameToSMB(sender);
		getNetBIOSDatagramService().setSourceName(srcName);
	}
	
	public void setReceiver(String receiver) {
		byte[] destName = SMBMessage.nameToSMB(receiver);
		getNetBIOSDatagramService().setDestinationName(destName);
	}
	
	public void setText(String sender, String receiver, String text) throws UnsupportedEncodingException {
		byte[] data = null;
		data = (sender + "\0" + receiver + "\0" + text + "\0").getBytes(outEncoding);		
		byte[] length = fromInt2ByteArray(data.length);
		getSMBMessageBlock().getTransRequest().setTotalDataCount(length);
		getSMBMessageBlock().getTransRequest().setDataCount(length);
		getSMBMessageBlock().getTransRequest().setByteCount(fromInt2ByteArray(data.length+TransRequest.BYTE_COUNT_CONST));
		
		getData().setData(data);
	}
	
	public String getSender() {
		return getFieldFromData(PopupSMBMessage.SENDER_FIELD);
	}
	
	public String getReceiver() {
		return getFieldFromData(PopupSMBMessage.RECEIVER_FIELD);
	}
	
	public String getText() {
		return getFieldFromData(PopupSMBMessage.TEXT_FIELD);
	}
	
	public byte[] getSenderIP() {
		return getNetBIOSDatagramService().getSourceIP();
	}
	
	private String getFieldFromData(int field) {
		byte[] data = getData().getBytes();
		String allText = null;
		try {
			allText = new String(data, inEncoding);
		} catch (UnsupportedEncodingException e) {
			System.err.println(e);
			// e.printStackTrace();
			JOptionPane.showMessageDialog(null,
					"Received message has unsupported encoding.\n" + e, "Warning",
					JOptionPane.WARNING_MESSAGE);
		}

		String[] fields = PopupSMBMessage.SEPARATOR_PATTERN.split(allText);
		return fields[field];
	}
	
	private byte[] fromInt2ByteArray(int val) {
		byte[] length = new byte[2];
		length[0] = (byte)(val % BIT_IN_BYTE);
		length[1] = (byte)(val / BIT_IN_BYTE);
		
		return length;
	}

	public static void setInEncoding(String inEncoding) {
		PopupSMBMessage.inEncoding = inEncoding;
	}

	public static String getInEncoding() {
		return inEncoding;
	}

	public static void setOutEncoding(String outEncoding) {
		PopupSMBMessage.outEncoding = outEncoding;
	}

	public static String getOutEncoding() {
		return outEncoding;
	}
}
