package jpopup.popupengine;

public class SMBMessage {
	public static final byte MAX_SMB_NAME_LENGTH = 16;
	public static final byte SMB_DESTINATION_NAME_BYTES_COUNT = 34;
	
	private NetBIOSDatagramService myNetBIOSDatagramService;
	private SMBMessageBlockProtocol mySMBMessageBlock;
	private SMBMessageData myData;
	
	public SMBMessage() {
		myNetBIOSDatagramService = new NetBIOSDatagramService();
		mySMBMessageBlock = new SMBMessageBlockProtocol();
		myData = new SMBMessageData();
	}
	
	public SMBMessage(byte[] myNetBIOSDatagramService, byte[] mySMBHeader, byte[] myTransRequest, byte[] myData) {
		this.myNetBIOSDatagramService = new NetBIOSDatagramService(myNetBIOSDatagramService);
		mySMBMessageBlock = new SMBMessageBlockProtocol(new SMBHeader(mySMBHeader), new TransRequest(myTransRequest));
		this.myData = new SMBMessageData(myData);
	}
	
	public SMBMessage(byte[] data) {
		int start = 14;
		int counter = start;
		int srcNameLength = 0;
		int destNameLength = 0;
		do {
			srcNameLength++;
			counter++;
		} while (data[counter - 1] != 0);
		
		do {
			destNameLength++;
			counter++;
		} while (data[counter - 1] != 0);
		
		start = 0;
		int length = 14 + srcNameLength + destNameLength;
		byte[] netBIOSDatagramService = new byte[length];
		System.arraycopy(data, start, netBIOSDatagramService, 0, length);
		
		start += length;
		length = 32;
		byte[] smbHeader = new byte[length];
		System.arraycopy(data, start, smbHeader, 0, length);
				
		start += length;
		length = 56;
		byte [] transRequest = new byte[length];
		System.arraycopy(data, start, transRequest, 0, length);
		
		start += length;
		length = data.length - start;
		byte[] msgData = new byte[length];
		System.arraycopy(data, start, msgData, 0, length);
		
		myNetBIOSDatagramService = new NetBIOSDatagramService(netBIOSDatagramService);
		mySMBMessageBlock = new SMBMessageBlockProtocol(smbHeader, transRequest);
		myData = new SMBMessageData(msgData);
	}


	public byte[] getBytes() {		
		byte[] netBIOSDatagramService = myNetBIOSDatagramService.getBytes();		
		byte[] smbMessageBlock = mySMBMessageBlock.getBytes();
		byte[] data = myData.getBytes();
		
		byte[] resultArray = new byte[netBIOSDatagramService.length + smbMessageBlock.length
		                          + data.length];		
		
		int destPos = 0;		
		System.arraycopy(netBIOSDatagramService, 0, resultArray, destPos, netBIOSDatagramService.length);
		destPos += netBIOSDatagramService.length;
		System.arraycopy(smbMessageBlock, 0, resultArray, destPos, smbMessageBlock.length);
		destPos += smbMessageBlock.length;
		System.arraycopy(data, 0, resultArray, destPos, data.length);
		
		return resultArray;
	}	
	
	protected void setNetBIOSDatagramService(NetBIOSDatagramService myNetBIOSDatagramService) {
		this.myNetBIOSDatagramService = myNetBIOSDatagramService;
	}

	protected NetBIOSDatagramService getNetBIOSDatagramService() {
		return myNetBIOSDatagramService;
	}

	protected void setSMBMessageBlock(SMBMessageBlockProtocol mySMBMessageBlock) {
		this.mySMBMessageBlock = mySMBMessageBlock;
	}

	protected SMBMessageBlockProtocol getSMBMessageBlock() {
		return mySMBMessageBlock;
	}

	public void setData(SMBMessageData myData) {
		this.myData = myData;
	}

	protected SMBMessageData getData() {
		return myData;
	}
	
	public static byte[] nameToSMB(String name) {		
		StringBuilder newName = new StringBuilder(name);
		for (int i = newName.length(); i < SMBMessage.MAX_SMB_NAME_LENGTH; i++) {
			newName.append(" ");
		}
		byte c;
	    byte[] HexName = new byte[SMBMessage.SMB_DESTINATION_NAME_BYTES_COUNT];
	    for (int i = 0; i < SMBMessage.MAX_SMB_NAME_LENGTH; i++) {
	        c = (byte) newName.toString().toUpperCase().charAt(i);
	        HexName[i * 2 + 1] = (byte) (((c >> 4) & 0x000F) + 'A');
	        HexName[(i * 2) + 2] = (byte) ((c & 0x000F) + 'A');
	    }
	    HexName[0] = 32;
	    HexName[SMB_DESTINATION_NAME_BYTES_COUNT - 3] = 65;
	    HexName[SMB_DESTINATION_NAME_BYTES_COUNT - 1] = '\0';
	    return HexName;   
	}
	
	public byte[] getHeader() {
		return mySMBMessageBlock.getSmbHeader().getBytes();
	}
	
}
