package jpopup.popupengine.text;

import java.awt.event.ActionEvent;
import java.io.IOException;
import java.net.DatagramPacket;
import java.net.DatagramSocket;
import java.net.InetAddress;
import java.net.UnknownHostException;
import java.util.HashMap;
import java.util.Iterator;

import jpopup.popupengine.JSMBEngine;
import jpopup.popupengine.PopupCatcher;
import jpopup.popupengine.PopupListener;
import jpopup.popupengine.PopupSMBMessage;

/**
 * This class used to post/get WinPopupMessages, cut long popup messages and make other handling of receiveing/sending Winpopup messages.
 * @author Alexey Kalukin
 *
 */
public class JPopupEngine extends JSMBEngine {
	private static final int UDP_PORT = 138;
	private static final String WORKGROUP_MASK = "10.255.255.255";

	public static final int MAX_POPUP_MESSAGE_LENGTH = 405;
	public static final int ZERO_BYTES_COUNT = 3;
	public static final String R_ARROW = "~-->~";
	public static final String L_ARROW = "~<--~";

	private HashMap<String, TextPopupMSG> bufferForLongMSG = new HashMap<String, TextPopupMSG>(); //If coming alot cutting msgs then may be memory overflow Need create some timer for cleaning msgs

	public JPopupEngine(PopupCatcher catcher) {
		super(catcher);
	}

	public boolean sendMessage(String sender, String receiver, byte[] data) {
		DatagramSocket socket = null;

		try {
			String text = new String(data, PopupSMBMessage.getOutEncoding());
			String[] msgs = new String[] { text };
			if ((sender + "\0" + receiver + "\0" + text + "\0").length() > JPopupEngine.MAX_POPUP_MESSAGE_LENGTH) {
				msgs = cutMsg(sender, receiver, text);
			}

			InetAddress address;			
			boolean isWorkGroup = false;
			for (Iterator<String> iterator = workGroupIterator(); iterator.hasNext();) {
				String workGroup = iterator.next();
				if (workGroup.equalsIgnoreCase(receiver)) {
					isWorkGroup = true;
				}			
			}
			if (isWorkGroup) {
				address = InetAddress.getByName(WORKGROUP_MASK);
			} else {
				address = InetAddress.getByName(receiver);
			}
			
			socket = new DatagramSocket();
			DatagramPacket packet = new DatagramPacket(new byte[1], 1, address,
					UDP_PORT);
			PopupSMBMessage popupMsg = new PopupSMBMessage(sender, receiver,
					msgs[0]);

			for (String msg : msgs) {
				popupMsg.setText(sender, receiver, msg);
				byte[] msgArr = popupMsg.getBytes();

				packet.setData(msgArr);
				packet.setLength(msgArr.length);
				socket.send(packet);
			}
		} catch (UnknownHostException uhe) {
			System.err.println(uhe);
		//	uhe.printStackTrace();
			return false;
		} catch (IOException ioe) {
			System.err.println(ioe);
		//	ioe.printStackTrace();
			return false;
		} finally {
			if (socket != null) {
				socket.close();
			}
		}

		return true;
	}

	public void actionPerformed(ActionEvent e) {
		TextPopupMSG popupMSG = (TextPopupMSG) e.getSource();

		if (isCutMsg(popupMSG.getText().toString())) {
			if (bufferForLongMSG.containsKey(popupMSG.getSender())) {
				bufferForLongMSG.get(popupMSG.getSender()).getText().append(
						clearCutMsg(popupMSG.getText().toString()));
			} else {
				popupMSG.setText(new StringBuilder(clearCutMsg(popupMSG
						.getText().toString())));
				bufferForLongMSG.put(popupMSG.getSender(), popupMSG);
			}
		} else {
			if (isEndOfCutMsg(popupMSG.getText().toString())
					&& bufferForLongMSG.containsKey(popupMSG.getSender())) {
				bufferForLongMSG.get(popupMSG.getSender()).getText().append(
						clearCutMsg(popupMSG.getText().toString()));
				popupMSG = bufferForLongMSG.get(popupMSG.getSender());
				bufferForLongMSG.remove(popupMSG.getSender());
			}

			for (Iterator<PopupListener> iterator = listenersIterator(); iterator.hasNext();) {
				PopupListener listener = iterator.next();
				listener.getPopupMSG(popupMSG);
			}
		}
	}

	private String[] cutMsg(String sender, String receiver, String text) {
		int msgLength = JPopupEngine.MAX_POPUP_MESSAGE_LENGTH
				- JPopupEngine.ZERO_BYTES_COUNT - sender.length()
				- receiver.length() - JPopupEngine.R_ARROW.length()
				- JPopupEngine.L_ARROW.length();

		int start = 0;
		int finish = start + msgLength;
		int msgCount = text.length() / msgLength;
		String[] msgs = new String[msgCount + 1];

		msgs[0] = text.substring(start, finish) + R_ARROW;

		start = finish;
		finish += msgLength;
		for (int i = 1; i < msgCount; i++) {
			msgs[i] = L_ARROW + text.substring(start, finish) + R_ARROW;
			start = finish;
			finish += msgLength;
		}

		msgs[msgCount] = L_ARROW + text.substring(start, text.length());

		return msgs;
	}

	private boolean isCutMsg(String text) {
		return (text.length() >= R_ARROW.length())
				&& (text.startsWith(R_ARROW, text.length() - R_ARROW.length()));
	}

	private boolean isEndOfCutMsg(String text) {
		return (text.length() > L_ARROW.length()) && (text.startsWith(L_ARROW));
	}

	private String clearCutMsg(String text) {
		String clearText = text;
		if (text.startsWith(L_ARROW)) {
			clearText = text.substring(L_ARROW.length(), text.length()
					- L_ARROW.length());
		}
		if (clearText
				.startsWith(R_ARROW, clearText.length() - R_ARROW.length())) {
			clearText = clearText.substring(0, clearText.length()
					- R_ARROW.length());
		}

		return clearText;
	}
}
