package plugins.spamfilters;

import java.io.File;
import java.io.IOException;
import java.util.ArrayList;
import java.util.List;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerException;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.dom.DOMSource;
import javax.xml.transform.stream.StreamResult;

import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import org.w3c.dom.Text;
import org.xml.sax.ErrorHandler;
import org.xml.sax.SAXException;
import org.xml.sax.SAXParseException;

public class FilterListManager {
	private static final String ROOT_TAG = "filters";
	private static final String FILTER_TAG = "filter";
	private static final String ACTIVE_TAG = "active";
	
	private DocumentBuilderFactory dbf;
	private DocumentBuilder parser;
	
	private MessageContainsFilter filter;	

	public FilterListManager(MessageContainsFilter filter) throws ParserConfigurationException {
		this.filter = filter;
		
		dbf = DocumentBuilderFactory.newInstance();
		dbf.setValidating(false);

		parser = dbf.newDocumentBuilder();

		parser.setErrorHandler(new ErrorHandler() {

			public void error(SAXParseException exception) throws SAXException {
				System.err.println("ERROR: " + exception.getMessage());
			}

			public void fatalError(SAXParseException exception)
					throws SAXException {
				System.err.println("FATAL: " + exception.getMessage());
				throw exception;
			}

			public void warning(SAXParseException exception)
					throws SAXException {
				System.err.println("WARNING: " + exception.getMessage());
			}

		});
	}
	
	public List<String> loadFilters(String XMLFileName) throws SAXException, IOException {
		Document xml = parser.parse(XMLFileName);
		List<String> filterList = getListFromXML(xml);
		
		NodeList isActiveNodes = xml.getElementsByTagName(ACTIVE_TAG);		
		String isActive = ((Text)isActiveNodes.item(0).getFirstChild()).getData().trim();
		filter.setActive(new Boolean(isActive));
				
		return filterList;
	}
	
	public void saveFilters(String xmlFileName) throws TransformerException {		
		Document xml = putFiltersToXML();

		TransformerFactory factory = TransformerFactory.newInstance();
		Transformer transformer = factory.newTransformer();
		transformer.transform(new DOMSource(xml), new StreamResult(new File(
				xmlFileName)));
	}
	
	private Document putFiltersToXML() {
		Document xml = parser.newDocument();
		xml.appendChild(xml.createElement(ROOT_TAG));
		Node rootNode = xml.getElementsByTagName(ROOT_TAG).item(0);

		for (String filter : this.filter) {
			putFilter2XML(rootNode, filter);
		}
		
		Node filterNode = rootNode.getOwnerDocument().createElement(ACTIVE_TAG);
		filterNode.appendChild(rootNode.getOwnerDocument().createTextNode(Boolean.toString(filter.isActive())));		
		rootNode.appendChild(filterNode);

		return xml;
	}
	
	private void putFilter2XML(Node parent, String filter) {
		Node filterNode = parent.getOwnerDocument().createElement(FILTER_TAG);
		filterNode.appendChild(parent.getOwnerDocument().createTextNode(filter));		

		parent.appendChild(filterNode);
	}
	
	private List<String> getListFromXML(Document xml) {
		NodeList filters = xml.getElementsByTagName(FILTER_TAG);
		List<String> filtersList = new ArrayList<String>();

		int filtersCount = filters.getLength();
		for (int i = 0; i < filtersCount; i++) {
			Element filter = (Element) filters.item(i);
			String newContact = ((Text)filter.getFirstChild()).getData().trim();
			filtersList.add(newContact);
		}		
		
		return filtersList;
	}
}
