package plugins.spamfilters.gui;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.GridLayout;

import javax.swing.BorderFactory;
import javax.swing.DefaultListModel;
import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JFrame;
import javax.swing.JList;
import javax.swing.JMenuBar;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.ListSelectionModel;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;
import javax.xml.parsers.ParserConfigurationException;

import jpopup.plugins.PluginEngine;

import plugins.spamfilters.FilterListManager;
import plugins.spamfilters.MessageContainsFilter;
import plugins.spamfilters.gui.actions.AddFilterAction;
import plugins.spamfilters.gui.actions.EditFilterAction;
import plugins.spamfilters.gui.actions.RemoveFilterAction;
import plugins.spamfilters.gui.listeners.ContainsFilterWindowListener;
import plugins.spamfilters.gui.listeners.FiltersListMouseListener;

public class ContainsMsgFilterWindow extends JFrame {
	private static final long serialVersionUID = -5861913673106688521L;

	private static final String WINDOW_TITLE = "Filter Options";
	
	private static final String ACTIVE_FILTER_STRING = "Set filter active";
	
	private final int INDENT = 5;
	
	private final int DEFAULT_WIDTH = 300;
	private final int DEFAULT_HEIGHT = 300;	
	private final int DEFAULT_EDIT_WIDTH = 100;
	private final int DEFAULT_EDIT_HEIGHT = 100;
	
	private String filterFileName = PluginEngine.PLUGINS_DIRECTORY + "/filters/filters.xml";
	private static String iconPath = "filters/icons";
	
	private MessageContainsFilter filter;

	private JMenuBar mainMenu;
	private JPanel mainPanel;
	private JPanel filterListPanel;
	private DefaultListModel filterListModel;
	private JList filtersList;
	private JPanel editFilterPanel;
	private JButton addFilterButton;
	private FilterListManager filterManager;
	private JCheckBox isActiveCheckBox;
	private JPanel activatePanel;
	private JButton editFilterButton;
	private JButton removeFilterButton;
	
	private static ImageIcon addFilterIcon = new ImageIcon(PluginEngine.PLUGINS_DIRECTORY + "/" + iconPath + "/" + "addFilter.gif");
	private static ImageIcon removeFilterIcon = new ImageIcon(PluginEngine.PLUGINS_DIRECTORY + "/" + iconPath + "/" + "removeFilter.gif");
	private static ImageIcon editFilterIcon = new ImageIcon(PluginEngine.PLUGINS_DIRECTORY + "/" + iconPath + "/" + "editFilter.gif");
	private static ImageIcon mainIcon = new ImageIcon(PluginEngine.PLUGINS_DIRECTORY + "/" + iconPath + "/" + "mainIcon.gif");
	
	public ContainsMsgFilterWindow(MessageContainsFilter filter) {		
		this.setFilter(filter);
		
		try {
			filterManager = new FilterListManager(filter);
		} catch (ParserConfigurationException e) {
			JOptionPane.showMessageDialog(null,
					"Error configure filter xml parser" + e, "Error", JOptionPane.ERROR_MESSAGE);
		//	e.printStackTrace();
		}
		setTitle(WINDOW_TITLE);		
		
		setDefaultCloseOperation(JFrame.DISPOSE_ON_CLOSE);		
		addWindowListener(new ContainsFilterWindowListener(this));
		
		mainMenu = createMainMenu();
		setJMenuBar(mainMenu);
		
		mainPanel = createMainPanel();
		getContentPane().add(mainPanel);
		
		setPreferredSize(new Dimension(DEFAULT_WIDTH, DEFAULT_HEIGHT));
		setIconImage(mainIcon.getImage());
	}	
	
	private JMenuBar createMainMenu() {
		return null;
	}
	
	private JPanel createMainPanel() {
		JPanel mainPanel = new JPanel(new BorderLayout(INDENT,INDENT));
		mainPanel.setBorder(BorderFactory.createEmptyBorder(INDENT,INDENT,INDENT,INDENT));
		
		filterListPanel = createFilterListPanel();		
		editFilterPanel = createEditFilterPanel();		
		activatePanel = createActivatePanel();
		
		JPanel editPanel = new JPanel(new BorderLayout());
		editPanel.add(editFilterPanel, BorderLayout.NORTH);		
		
		mainPanel.add(filterListPanel, BorderLayout.CENTER);
		mainPanel.add(editPanel, BorderLayout.EAST);
		mainPanel.add(activatePanel, BorderLayout.SOUTH);
		
		return mainPanel;
	}
	
	private JPanel createEditFilterPanel() {
		JPanel editFilterPanel = new JPanel(new GridLayout(0,1,INDENT, INDENT));
		editFilterPanel.setPreferredSize(new Dimension(DEFAULT_EDIT_WIDTH, DEFAULT_EDIT_HEIGHT));
		
		addFilterButton = new JButton(new AddFilterAction(this));
		addFilterButton.setIcon(addFilterIcon);
		
		editFilterButton = new JButton(new EditFilterAction(this));
		editFilterButton.setIcon(editFilterIcon);
		editFilterButton.setEnabled(false);
		
		removeFilterButton = new JButton(new RemoveFilterAction(this));
		removeFilterButton.setIcon(removeFilterIcon);
		removeFilterButton.setEnabled(false);
		
		editFilterPanel.add(addFilterButton);
		editFilterPanel.add(editFilterButton);
		editFilterPanel.add(removeFilterButton);
		
		return editFilterPanel;
	}
	
	private JPanel createFilterListPanel() {
		JPanel filtersListPanel = new JPanel(new BorderLayout());
		
		filterListModel = new DefaultListModel();
		getFilters();
		filtersList = new JList(filterListModel);
		filtersList.addMouseListener(new FiltersListMouseListener(this));
		filtersList.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
		filtersList.addListSelectionListener(new ListSelectionListener() {

			public void valueChanged(ListSelectionEvent event) {
				if (event.getFirstIndex() > -1) {
					editFilterButton.setEnabled(true);
					removeFilterButton.setEnabled(true);
				} else {
					editFilterButton.setEnabled(false);
					removeFilterButton.setEnabled(false);
				}
				
			}
			
		});
		
		JScrollPane filterListScrollPane = new JScrollPane(filtersList);		
		
		filtersListPanel.add(filterListScrollPane);
		
		return filtersListPanel;
	}
	
	private JPanel createActivatePanel() {
		FlowLayout layout = new FlowLayout(FlowLayout.LEFT);
		layout.setHgap(INDENT);
		layout.setVgap(INDENT);
		
		JPanel activatePanel = new JPanel(layout);
		
		isActiveCheckBox = new JCheckBox(ACTIVE_FILTER_STRING);
		isActiveCheckBox.addChangeListener(new ChangeListener() {
			public void stateChanged(ChangeEvent event) {
				JCheckBox checkBox = (JCheckBox)event.getSource();
				
				getFilter().setActive(checkBox.isSelected());
			}
		});
		isActiveCheckBox.setSelected(filter.isActive());
		
		activatePanel.add(isActiveCheckBox);
		
		return activatePanel;
	}
		
	private void getFilters() {
		for (String filterString: getFilter()) {
			filterListModel.addElement(filterString);
		}
	}
	
	public void setFilter(MessageContainsFilter filter) {
		this.filter = filter;
	}

	public MessageContainsFilter getFilter() {
		return filter;
	}

	public static void setMainIcon(ImageIcon mainIcon) {
		ContainsMsgFilterWindow.mainIcon = mainIcon;
	}

	public static ImageIcon getMainIcon() {
		return mainIcon;
	}

	public void setFilterListModel(DefaultListModel filterListModel) {
		this.filterListModel = filterListModel;
	}

	public DefaultListModel getFilterListModel() {
		return filterListModel;
	}

	public void setFilterFileName(String filterFileName) {
		this.filterFileName = filterFileName;
	}

	public String getFilterFileName() {
		return filterFileName;
	}

	public void setFilterManager(FilterListManager filterManager) {
		this.filterManager = filterManager;
	}

	public FilterListManager getFilterManager() {
		return filterManager;
	}

	public void setIsActiveCheckBox(JCheckBox isActiveCheckBox) {
		this.isActiveCheckBox = isActiveCheckBox;
	}

	public JCheckBox getIsActiveCheckBox() {
		return isActiveCheckBox;
	}
	
	public static void setAddFilterIcon(ImageIcon addFilterIcon) {
		ContainsMsgFilterWindow.addFilterIcon = addFilterIcon;
	}

	public static ImageIcon getAddFilterIcon() {
		return addFilterIcon;
	}

	public static void setRemoveFilterIcon(ImageIcon removeFilterIcon) {
		ContainsMsgFilterWindow.removeFilterIcon = removeFilterIcon;
	}

	public static ImageIcon getRemoveFilterIcon() {
		return removeFilterIcon;
	}

	public static void setEditFilterIcon(ImageIcon editFilterIcon) {
		ContainsMsgFilterWindow.editFilterIcon = editFilterIcon;
	}

	public static ImageIcon getEditFilterIcon() {
		return editFilterIcon;
	}
	
	public void setFiltersList(JList filtersList) {
		this.filtersList = filtersList;
	}

	public JList getFiltersList() {
		return filtersList;
	}
	
	public void setRemoveFilterButton(JButton removeFilterButton) {
		this.removeFilterButton = removeFilterButton;
	}

	public JButton getRemoveFilterButton() {
		return removeFilterButton;
	}

	public void setEditFilterButton(JButton editFilterButton) {
		this.editFilterButton = editFilterButton;
	}

	public JButton getEditFilterButton() {
		return editFilterButton;
	}
	
	
}
